/* axis.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_AXIS_H)
#define	_AXIS_H

/* Axis handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIAxis *MPIAxis;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "filter.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* Axis Config */
typedef struct MPIAxisInPosition {
	struct {
		float	positionFine;
		long	positionCoarse;
		float	velocity;
	} tolerance;
	float	settlingTime;	/* seconds */
	long	settleOnStop;
	long	settleOnEstop;
	long	settleOnEstopCmdEqAct;
} MPIAxisInPosition;

typedef enum {
	MPIAxisMasterTypeINVALID = -1,

	MPIAxisMasterTypeNONE,
	MPIAxisMasterTypeMOTOR_FEEDBACK_PRIMARY,
	MPIAxisMasterTypeMOTOR_FEEDBACK_SECONDARY,
	MPIAxisMasterTypeAXIS_COMMANDED_POSITION,
	MPIAxisMasterTypeAXIS_ACTUAL_POSITION,
	MPIAxisMasterTypeADDRESS,

	MPIAxisMasterTypeLAST,
	MPIAxisMasterTypeFIRST = MPIAxisMasterTypeINVALID + 1
} MPIAxisMasterType;

typedef struct MPIAxisMaster {
	MPIAxisMasterType	type;
	long				number;
	long				*address;
	long				encoderFaultMotorNumber;
} MPIAxisMaster;

typedef struct MPIAxisEstopModify {
	float	deceleration;
	float	decelerationJerk;
	float	jerkPercent;
} MPIAxisEstopModify;

typedef struct MPIAxisConfig {
	MPIAxisEstopModify		estopModify;

	MPIAxisInPosition		inPosition;
	MPIAxisMaster			master;
	long					masterCorrection;
							
	MPIObjectMap			filterMap;
} MPIAxisConfig;

typedef enum {
	MPIAxisMessageFIRST = mpiMessageID(MPIModuleIdAXIS, 0),

	MPIAxisMessageAXIS_INVALID,
	MPIAxisMessageCOMMAND_NOT_SET,
	MPIAxisMessageNOT_MAPPED_TO_MS,

	MPIAxisMessageLAST
} MPIAxisMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIAxis MPI_DECL2
	mpiAxisCreate(MPIControl	control,
				  long			number);

MPI_DECL1 long MPI_DECL2
	mpiAxisDelete(MPIAxis axis);

MPI_DECL1 long MPI_DECL2
	mpiAxisValidate(MPIAxis axis);

/* Axis */
MPI_DECL1 long MPI_DECL2
	mpiAxisActualPositionGet(MPIAxis	axis,
							 double		*actual);

MPI_DECL1 long MPI_DECL2
	mpiAxisActualPositionGet32(MPIAxis	axis,
							 double		*actual);

MPI_DECL1 long MPI_DECL2
	mpiAxisActualPositionSet(MPIAxis	axis,
							 double		actual);

MPI_DECL1 long MPI_DECL2
	mpiAxisActualVelocity(MPIAxis	axis,
						  double	*actual);

MPI_DECL1 long MPI_DECL2
	mpiAxisCommandPositionGet(MPIAxis	axis,
							  double	*command);

MPI_DECL1 long MPI_DECL2
	mpiAxisCommandPositionGet32(MPIAxis	axis,
							 double		*command);

MPI_DECL1 long MPI_DECL2
	mpiAxisCommandPositionSet(MPIAxis	axis,
							  double	command);

MPI_DECL1 long MPI_DECL2
	mpiAxisConfigGet(MPIAxis		axis,
					 MPIAxisConfig	*config,
					 void			*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisConfigSet(MPIAxis		axis,
					 MPIAxisConfig	*config,
					 void			*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiAxisControl(MPIAxis axis);

MPI_DECL1 long MPI_DECL2
	mpiAxisEventNotifyGet(MPIAxis		axis,
						  MPIEventMask	*eventMask,
						  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisEventNotifySet(MPIAxis		axis,
						  MPIEventMask	eventMask,
						  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisEventReset(MPIAxis		axis,
					  MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiAxisFlashConfigGet(MPIAxis		axis,
						  void			*flash,
						  MPIAxisConfig	*config,
						  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisFlashConfigSet(MPIAxis		axis,
						  void			*flash,
						  MPIAxisConfig	*config,
						  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisMemory(MPIAxis	axis,
				  void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiAxisMemoryGet(MPIAxis	axis,
					 void		*dst,
					 const void	*src,
					 long		count);

MPI_DECL1 long MPI_DECL2
	mpiAxisMemorySet(MPIAxis	axis,
					 void		*dst,
					 const void	*src,
					 long		count);

MPI_DECL1 long MPI_DECL2
	mpiAxisMotorMapGet(MPIAxis		axis,
					   MPIObjectMap	*motorMap);

MPI_DECL1 long MPI_DECL2
	mpiAxisNumber(MPIAxis	axis,
				  long		*number);

MPI_DECL1 long MPI_DECL2
	mpiAxisOriginGet(MPIAxis	axis,
					 double		*origin);

MPI_DECL1 long MPI_DECL2
	mpiAxisOriginSet(MPIAxis	axis,
					 double		origin);

MPI_DECL1 long MPI_DECL2
	mpiAxisFlashOriginGet(MPIAxis	axis,
					 void		*flash,
					 double		*origin);

MPI_DECL1 long MPI_DECL2
	mpiAxisFlashOriginSet(MPIAxis	axis,
					 void		*flash,
					 double		origin);

MPI_DECL1 long MPI_DECL2
	mpiAxisPositionError(MPIAxis	axis,
						 double		*error);	

MPI_DECL1 long MPI_DECL2
	mpiAxisFilterMapGet(MPIAxis		axis,
						  MPIObjectMap	*filterMap);

MPI_DECL1 long MPI_DECL2
	mpiAxisFilterMapSet(MPIAxis		axis,
						  MPIObjectMap	filterMap);

MPI_DECL1 long MPI_DECL2
	mpiAxisStatus(MPIAxis	axis,
				  MPIStatus	*status,
				  void		*external);

MPI_DECL1 long MPI_DECL2
	mpiAxisTrajectory(MPIAxis		axis,
					  MPITrajectory	*trajectory);

#if defined(__cplusplus)
}
#endif

#endif
