/* filter.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_FILTER_H)
#define	_FILTER_H

/* Filter handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIFilter *MPIFilter;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* Filter Config */
typedef union {
	float	f;
	long	l;
} MPIFilterCoeff;

#define	MPIFilterCoeffCOUNT_MAX	(20)

typedef struct MPIFilterGain {
	MPIFilterCoeff	coeff[MPIFilterCoeffCOUNT_MAX];
} MPIFilterGain;

#define	MPIFilterGainCOUNT_MAX	(5)

typedef struct MPIFilterConfig {
	long			gainIndex;
	MPIFilterGain	gain[MPIFilterGainCOUNT_MAX];

	MPIObjectMap	axisMap;
	MPIObjectMap	motorMap;
} MPIFilterConfig;

typedef enum {
	MPIFilterMessageFIRST = mpiMessageID(MPIModuleIdFILTER, 0),

	MPIFilterMessageFILTER_INVALID,
	MPIFilterMessageINVALID_ALGORITHM,
	MPIFilterMessageINVALID_DRATE,
	MPIFilterMessageCONVERSION_DIV_BY_0,
	MPIFilterMessageSECTION_NOT_ENABLED,
	MPIFilterMessageINVALID_FILTER_FORM,
	MPIFilterMessageINVALID_KA1,
	
	MPIFilterMessageLAST
} MPIFilterMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIFilter MPI_DECL2
	mpiFilterCreate(MPIControl	control,
					long		number);

MPI_DECL1 long MPI_DECL2
	mpiFilterDelete(MPIFilter filter);

MPI_DECL1 long MPI_DECL2
	mpiFilterValidate(MPIFilter filter);

/* Filter */
MPI_DECL1 long MPI_DECL2
	mpiFilterAxisMapGet(MPIFilter		filter,
						MPIObjectMap	*axisMap);

MPI_DECL1 long MPI_DECL2
	mpiFilterAxisMapSet(MPIFilter		filter,
						MPIObjectMap	axisMap);

MPI_DECL1 long MPI_DECL2
	mpiFilterConfigGet(MPIFilter		filter,
					   MPIFilterConfig	*config,
					   void				*external);

MPI_DECL1 long MPI_DECL2
	mpiFilterConfigSet(MPIFilter		filter,
					   MPIFilterConfig	*config,
					   void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiFilterControl(MPIFilter filter);

MPI_DECL1 long MPI_DECL2
	mpiFilterFlashConfigGet(MPIFilter		filter,
							void			*flash,
							MPIFilterConfig	*config,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiFilterFlashConfigSet(MPIFilter		filter,
							void			*flash,
							MPIFilterConfig	*config,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiFilterGainGet(MPIFilter		filter,
					 long			gainIndex,
					 MPIFilterGain	*gain);

MPI_DECL1 long MPI_DECL2
	mpiFilterGainIndexGet(MPIFilter	filter,
						  long		*gainIndex);

MPI_DECL1 long MPI_DECL2
	mpiFilterGainIndexSet(MPIFilter	filter,
						  long		gainIndex);

MPI_DECL1 long MPI_DECL2
	mpiFilterGainSet(MPIFilter		filter,
					 long			gainIndex,
					 MPIFilterGain	*gain);

MPI_DECL1 long MPI_DECL2
	mpiFilterIntegratorReset(MPIFilter filter);

MPI_DECL1 long MPI_DECL2
	mpiFilterMemory(MPIFilter	filter,
					void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiFilterMemoryGet(MPIFilter	filter,
					   void			*dst,
					   const void	*src,
					   long			count);

MPI_DECL1 long MPI_DECL2
	mpiFilterMemorySet(MPIFilter	filter,
					   void			*dst,
					   const void	*src,
					   long			count);

MPI_DECL1 long MPI_DECL2
	mpiFilterMotorMapGet(MPIFilter		filter,
						 MPIObjectMap	*motorMap);

MPI_DECL1 long MPI_DECL2
	mpiFilterMotorMapSet(MPIFilter		filter,
						 MPIObjectMap	motorMap);

MPI_DECL1 long MPI_DECL2
	mpiFilterNumber(MPIFilter	filter,
					long		*number);

#if defined(__cplusplus)
}
#endif

#endif
