/* flash.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_FLASH_H)
#define	_FLASH_H

/* Flash handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIFlash *MEIFlash;
#if defined(__cplusplus)
}
#endif

#include "meidef.h"
#include "control.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct MEIFlashSection {
	unsigned char	*address;
	long			size;
	long			sectorIndex;
} MEIFlashSection;

typedef struct MEIFlashConfig {
	long			wordSize;
	long			sectorSize;
	long			sectors;
	long			firmwareSectors;
	long			fpgaSectors;
	long			extMemSize;
	MEIFlashSection	all;
	MEIFlashSection	code;
	MEIFlashSection	codeBoot0;
	MEIFlashSection	codeBoot;
	MEIFlashSection	codeMain;
	MEIFlashSection	data;
	MEIFlashSection	dataExt;
	MEIFlashSection synqNet;
	MEIFlashSection misc;
	MEIFlashSection	FPGA0;
} MEIFlashConfig;

typedef enum {
	MEIFlashFileTypeNONE = 0,
	MEIFlashFileTypeCode,
	MEIFlashFileTypeDataInt,
	MEIFlashFileTypeDataExt,
	MEIFlashFileTypeSynqNet,
	MEIFlashFileTypeCodeAndData,
	MEIFlashFileTypeFPGA0,
	MEIFlashFileTypeALL		/* Loads Code and all FPGAs (for .bin files that include the FPGA images) */
} MEIFlashFileType;

typedef enum {
	MEIFlashMessageFIRST = mpiMessageID(MEIModuleIdFLASH, 0),

	MEIFlashMessageFLASH_INVALID,
	MEIFlashMessageFLASH_VERIFY_ERROR,
	MEIFlashMessageFLASH_WRITE_ERROR,
	MEIFlashMessagePATH,
	MEIFlashMessageNETWORK_TOPOLOGY_ERROR,

	MEIFlashMessageLAST
} MEIFlashMessage;

typedef enum {
	MEIFlashTraceFIRST = MEITraceLAST << 1,

	MEIFlashTraceLAST  = MEIFlashTraceFIRST << 1
} MEIFlashTrace;

/* Create/Delete/Validate */
MPI_DECL1 MEIFlash MPI_DECL2
	meiFlashCreate(MPIControl control);

MPI_DECL1 long MPI_DECL2
	meiFlashDelete(MEIFlash flash);

MPI_DECL1 long MPI_DECL2
	meiFlashValidate(MEIFlash flash);

/* Flash */
MPI_DECL1 long MPI_DECL2
	meiFlashConfigGet(MEIFlash			flash,
					  MEIFlashConfig	*config);

MPI_DECL1 long MPI_DECL2
	meiFlashConfigSet(MEIFlash			flash,
					  MEIFlashConfig	*config);

MPI_DECL1 MPIControl MPI_DECL2
	meiFlashControl(MEIFlash	flash);

MPI_DECL1 long MPI_DECL2
	meiFlashDataGet(MEIFlash	flash,
					void		*dst,
					void		*src,
					long		count);

MPI_DECL1 long MPI_DECL2
	meiFlashDataSet(MEIFlash	flash,
					void		*dst,
					void		*src,
					long		count);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryFromFileType(MEIFlash			flash,
							   const char		*fileName,
							   MEIFlashFileType fileType);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryFromFileImage(MEIFlash			flash,
							    const unsigned char	*fileImage,
							    MEIFlashFileType	fileType);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryGet(MEIFlash	 flash,
					  void		 *dst,
					  const void *src,
					  long		 count);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryModified(MEIFlash	flash,
						   long		*modified);

MPI_DECL1 long MPI_DECL2
	meiFlashMemorySet(MEIFlash	 flash,
					  void		 *dst,
					  const void *src,
					  long		 count);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryToFile(MEIFlash	flash,
						 const char	*fileName,
						 MEIFlashFileType fileType);

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryVerify(MEIFlash	flash,
						 const char	*fileName,
						 MEIFlashFileType fileType);

#if defined(__cplusplus)
}
#endif

#endif
