/* motor.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_MOTOR_H)
#define	_MOTOR_H

/* Motor handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIMotor *MPIMotor;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "event.h"
#include "platform.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {
	MPIMotorTypeINVALID = -1,

	MPIMotorTypeSERVO,
	MPIMotorTypeSTEPPER,

	MPIMotorTypeLAST,
	MPIMotorTypeFIRST = MPIMotorTypeINVALID + 1
} MPIMotorType;

typedef enum MPIMotorEncoder {
	MPIMotorEncoderINVALID = -1,

	MPIMotorEncoderPRIMARY,
	MPIMotorEncoderSECONDARY,

	MPIMotorEncoderLAST,
	MPIMotorEncoderFIRST = MPIMotorEncoderINVALID + 1
} MPIMotorEncoder;

typedef enum {
	MPIMotorEncoderFaultINVALID = -1,

	MPIMotorEncoderFaultPRIMARY,
	MPIMotorEncoderFaultSECONDARY,
	MPIMotorEncoderFaultPRIMARY_OR_SECONDARY,

	MPIMotorEncoderFaultLAST,
	MPIMotorEncoderFaultFIRST = MPIMotorEncoderFaultINVALID + 1
} MPIMotorEncoderFault;

typedef enum {
	MPIMotorDedicatedInINVALID			= -1,

	MPIMotorDedicatedInAMP_FAULT		= 0,
	MPIMotorDedicatedInBRAKE_APPLIED	= 1,
	MPIMotorDedicatedInHOME				= 2,
	MPIMotorDedicatedInLIMIT_HW_POS		= 3,
	MPIMotorDedicatedInLIMIT_HW_NEG		= 4,
	MPIMotorDedicatedInINDEX_PRIMARY	= 5,
	MPIMotorDedicatedInFEEDBACK_FAULT	= 6,
	MPIMotorDedicatedInCAPTURED			= 7,
	MPIMotorDedicatedInHALL_A			= 8,
	MPIMotorDedicatedInHALL_B			= 9,
	MPIMotorDedicatedInHALL_C			= 10,
	MPIMotorDedicatedInAMP_ACTIVE		= 11,
	MPIMotorDedicatedInINDEX_SECONDARY	= 12,
	MPIMotorDedicatedInWARNING			= 13,
	MPIMotorDedicatedInDRIVE_STATUS_9	= 14,
	MPIMotorDedicatedInDRIVE_STATUS_10	= 15,
	MPIMotorDedicatedInFEEDBACK_FAULT_PRIMARY	= 16,
	MPIMotorDedicatedInFEEDBACK_FAULT_SECONDARY	= 17,

	MPIMotorDedicatedInLAST,
	MPIMotorDedicatedInFIRST = MPIMotorDedicatedInINVALID + 1
} MPIMotorDedicatedIn;

typedef enum {
	MPIMotorDedicatedOutINVALID			= -1,

	MPIMotorDedicatedOutAMP_ENABLE		= 0,
	MPIMotorDedicatedOutBRAKE_RELEASE	= 1,

	MPIMotorDedicatedOutLAST,
	MPIMotorDedicatedOutFIRST = MPIMotorDedicatedOutINVALID + 1
} MPIMotorDedicatedOut;

typedef enum MPIMotorGeneralIo {
	MPIMotorGeneralIoINVALID = -1,

	MPIMotorGeneralIo0,
	MPIMotorGeneralIo1,
	MPIMotorGeneralIo2,
	MPIMotorGeneralIo3,
	MPIMotorGeneralIo4,
	MPIMotorGeneralIo5,
	MPIMotorGeneralIo6,
	MPIMotorGeneralIo7,
	MPIMotorGeneralIo8,
	MPIMotorGeneralIo9,
	MPIMotorGeneralIo10,
	MPIMotorGeneralIo11,
	MPIMotorGeneralIo12,
	MPIMotorGeneralIo13,
	MPIMotorGeneralIo14,
	MPIMotorGeneralIo15,
	MPIMotorGeneralIo16,
	MPIMotorGeneralIo17,
	MPIMotorGeneralIo18,
	MPIMotorGeneralIo19,
	MPIMotorGeneralIo20,
	MPIMotorGeneralIo21,
	MPIMotorGeneralIo22,
	MPIMotorGeneralIo23,
	MPIMotorGeneralIo24,
	MPIMotorGeneralIo25,
	MPIMotorGeneralIo26,
	MPIMotorGeneralIo27,
	MPIMotorGeneralIo28,
	MPIMotorGeneralIo29,
	MPIMotorGeneralIo30,
	MPIMotorGeneralIo31,

	MPIMotorGeneralIoLAST,
	MPIMotorGeneralIoFIRST = MPIMotorGeneralIoINVALID + 1
} MPIMotorGeneralIo;

typedef union {
	long					polarity;	/* 0 => active low, else active high */
	double					position;	/* MPIEventTypeLIMIT_SW_[POS|NEG] */
	float					error;		/* MPIEventTypeLIMIT_ERROR */
	MPIMotorEncoderFault	encoder;   	/* MPIEventTypeENCODER_FAULT */
} MPIMotorEventTrigger;

typedef struct MPIMotorEventConfig {
	MPIAction				action;
	MPIMotorEventTrigger	trigger;
	MPI_BOOL				direction;
	float					duration;	/* seconds */
} MPIMotorEventConfig;

typedef enum{
	MPIMotorBrakeModeINVALID = -1,

	MPIMotorBrakeModeNONE,
	MPIMotorBrakeModeDELAY,

	MPIMotorBrakeModeLAST,
	MPIMotorBrakeModeFIRST = MPIMotorBrakeModeINVALID + 1
} MPIMotorBrakeMode;

typedef struct MPIMotorBrake {
	MPIMotorBrakeMode	mode;
	float				applyDelay;
	float				releaseDelay;
} MPIMotorBrake;

typedef struct MPIMotorConfig {
	MPIMotorType		type;

	/* Event configuration, ordered by MPIEventType */
	MPIMotorEventConfig	event[MPIEventTypeMOTOR_LAST];

	float			abortDelay;
	float			enableDelay;
	MPIMotorBrake	brake;

	MPIObjectMap	filterMap;

} MPIMotorConfig;

typedef double MPIMotorFeedback[MPIMotorEncoderLAST];

typedef enum {
	MPIMotorMessageFIRST = mpiMessageID(MPIModuleIdMOTOR, 0),

	MPIMotorMessageMOTOR_INVALID,
	MPIMotorMessageTYPE_INVALID,
	MPIMotorMessageSTEPPER_NA,

	MPIMotorMessageLAST
} MPIMotorMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIMotor MPI_DECL2
	mpiMotorCreate(MPIControl	control,
				   long			number);

MPI_DECL1 long MPI_DECL2
	mpiMotorDelete(MPIMotor motor);

MPI_DECL1 long MPI_DECL2
	mpiMotorValidate(MPIMotor motor);

/* Motor */
MPI_DECL1 long MPI_DECL2
	mpiMotorAmpEnableGet(MPIMotor	motor,
						 MPI_BOOL  *ampEnable);

MPI_DECL1 long MPI_DECL2
	mpiMotorAmpEnableSet(MPIMotor	motor,
						 MPI_BOOL	ampEnable);

MPI_DECL1 long MPI_DECL2
	mpiMotorAxisMapGet(MPIMotor		motor,
					   MPIObjectMap	*axisMap);

MPI_DECL1 long MPI_DECL2
	mpiMotorConfigGet(MPIMotor			motor,
					  MPIMotorConfig	*config,
					  void				*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorConfigSet(MPIMotor			motor,
					  MPIMotorConfig	*config,
					  void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiMotorControl(MPIMotor motor);

MPI_DECL1 long MPI_DECL2
	mpiMotorEventConfigGet(MPIMotor				motor,
						   MPIEventType			eventType,
						   MPIMotorEventConfig	*eventConfig,
						   void					*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorEventConfigSet(MPIMotor				motor,
						   MPIEventType			eventType,
						   MPIMotorEventConfig	*eventConfig,
						   void					*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorEventNotifyGet(MPIMotor		motor,
						   MPIEventMask	*eventMask,
						   void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorEventNotifySet(MPIMotor		motor,
						   MPIEventMask	eventMask,
						   void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorEventReset(MPIMotor		motor,
					   MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiMotorFeedback(MPIMotor			motor,
					 MPIMotorFeedback	*feedback);

MPI_DECL1 long MPI_DECL2
	mpiMotorFilterMapGet(MPIMotor		motor,
						 MPIObjectMap	*filterMap);

MPI_DECL1 long MPI_DECL2
	mpiMotorFilterMapSet(MPIMotor		motor,
						 MPIObjectMap	filterMap);

MPI_DECL1 long MPI_DECL2
	mpiMotorFlashConfigGet(MPIMotor			motor,
						   void				*flash,
						   MPIMotorConfig	*config,
						   void				*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorFlashConfigSet(MPIMotor			motor,
						   void				*flash,
						   MPIMotorConfig	*config,
						   void				*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorDedicatedIn(MPIMotor		motor,
	                    long			startBit,
						long			bitCount,
						unsigned long	*state );

MPI_DECL1 long MPI_DECL2
	mpiMotorDedicatedOutGet(MPIMotor		motor,
	                        long			startBit,
							long			bitCount,
							unsigned long	*state );

MPI_DECL1 long MPI_DECL2
	mpiMotorGeneralIn(MPIMotor		motor,
	                  long			startBit,
					  long			bitCount,
					  unsigned long	*state );

MPI_DECL1 long MPI_DECL2
	mpiMotorGeneralOutGet(MPIMotor		motor,
	                      long			startBit,
						  long			bitCount,
						  unsigned long	*state );

MPI_DECL1 long MPI_DECL2
	mpiMotorGeneralOutSet(MPIMotor		motor,
	                      long			startBit,
						  long			bitCount,
						  unsigned long	state,
						  MPI_BOOL		wait );

MPI_DECL1 long MPI_DECL2
	mpiMotorMemory(MPIMotor	motor,
				   void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiMotorMemoryGet(MPIMotor	 motor,
					  void		 *dst,
					  const void *src,
					  long		 count);

MPI_DECL1 long MPI_DECL2
	mpiMotorMemorySet(MPIMotor	 motor,
					  void		 *dst,
					  const void *src,
					  long		 count);

MPI_DECL1 long MPI_DECL2
	mpiMotorNumber(MPIMotor	motor,
				   long		*number);

MPI_DECL1 long MPI_DECL2
	mpiMotorStatus(MPIMotor		motor,
				   MPIStatus	*status,
				   void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotorPositionFeedbackOffsetGet(MPIMotor	motor,
								      double		*offset);

MPI_DECL1 long MPI_DECL2
	mpiMotorPositionFeedbackOffsetSet(MPIMotor	motor,
							          double		offset);


#if defined(__cplusplus)
}
#endif

#endif
