/* mpidef.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_MPIDEF_H)
#define _MPIDEF_H

/*
 *	This file declares constants, typedefs, etc.
 *	used by the Motion Programming Interface.
 *
 *	This file is included by all MPI header files.
 */

/* MPI declaration prefixes (header) */
#if !defined(MPI_DECL1)
#define	MPI_DECL1
#endif

#if !defined(MPI_DECL2)
#define	MPI_DECL2
#endif

/* MPI definition prefixes (source) */
#if !defined(MPI_DEF1)
#define	MPI_DEF1
#endif

#if !defined(MPI_DEF2)
#define	MPI_DEF2
#endif

#if !defined(FALSE)
#define	FALSE	(0)
#endif

#if !defined(TRUE)
#define	TRUE	(! FALSE)
#endif

#if !defined(NULL)
#define	NULL	(0)
#endif

/* MPI Boolean helpers */
#ifndef MPI_BOOL
#define MPI_BOOL	long
#endif
#ifndef MPI_IS_TRUE
#define MPI_IS_TRUE(b)	((b) != FALSE)
#endif
#ifndef MPI_IS_FALSE
#define MPI_IS_FALSE(b)	((b) == FALSE)
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* MPIAction */
typedef enum {
	MPIActionINVALID = -1,

	MPIActionNONE,
	MPIActionSTOP,
	MPIActionE_STOP,
	MPIActionE_STOP_ABORT,
	MPIActionE_STOP_MODIFY,
	MPIActionE_STOP_CMD_EQ_ACT,
	MPIActionABORT,

	MPIActionDONE,
	MPIActionSTART,
	MPIActionRESUME,
	MPIActionRESET,
	MPIActionCANCEL_REPEAT,

	MPIActionLAST,
	MPIActionFIRST = MPIActionINVALID + 1
} MPIAction;

/* MPIEmpty */
typedef struct MPIEmpty {
	long	dummy;
} MPIEmpty;

/* MPIModuleID */
typedef enum {
	MPIModuleIdINVALID = -1,

	MPIModuleIdMESSAGE,
	MPIModuleIdAXIS,
	MPIModuleIdCAPTURE,
	MPIModuleIdCOMMAND,
	MPIModuleIdCOMPARE,
	MPIModuleIdCOMPENSATOR,
	MPIModuleIdCONTROL,
	MPIModuleIdEVENT,
	MPIModuleIdEVENTMGR,
	MPIModuleIdFILTER,
	MPIModuleIdMOTION,
	MPIModuleIdMOTOR,
	MPIModuleIdNOTIFY,
	MPIModuleIdPATH,
	MPIModuleIdPROBE,
	MPIModuleIdRECORDER,
	MPIModuleIdSEQUENCE,

	MPIModuleIdLAST,
	MPIModuleIdFIRST = MPIModuleIdINVALID + 1,

	MPIModuleIdEXTERNAL = 0x80,

	MPIModuleIdMAX = 0xFF
} MPIModuleId;

/* MPIState */
typedef enum {
	MPIStateINVALID = -1,

	MPIStateIDLE,
	MPIStateMOVING,
	MPIStateSTOPPING,
	MPIStateSTOPPED,
	MPIStateSTOPPING_ERROR,
	MPIStateERROR,

	MPIStateLAST,
	MPIStateFIRST = MPIStateINVALID + 1
} MPIState;

/* MPIWait */
typedef enum {
	MPIWaitFOREVER = -1,
	MPIWaitPOLL = 0,
	MPIWaitMSEC
} MPIWait;

#if defined(__cplusplus)
}
#endif

#include "object.h"
#include "message.h"

#include "event.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* MPIIoSource */
typedef union {
	MPIHandle	motor;	/* MOTOR */
	long		index;	/* USER */
} MPIIoSource;

/* MPIIoType */
typedef enum {
	MPIIoTypeINVALID = -1,

	MPIIoTypeMOTOR_DEDICATED,
	MPIIoTypeMOTOR_GENERAL,
	MPIIoTypeUSER,

	MPIIoTypeLAST,
	MPIIoTypeFIRST = MPIIoTypeINVALID + 1
} MPIIoType;

/* MPIIoTrigger */
typedef struct MPIIoTrigger {
	MPIIoType			type;
	MPIIoSource			source;
	unsigned long		mask;
	unsigned long		pattern;
} MPIIoTrigger;

/* MPIStatus */
typedef enum {
	MPIStatusFlagINVALID = -1,


	MPIStatusFlagLAST,
	MPIStatusFlagFIRST = MPIStatusFlagINVALID + 1,

	MPIStatusFlagMOTOR_FIRST	= MPIStatusFlagINVALID,
	MPIStatusFlagMOTOR_LAST		= MPIStatusFlagLAST
} MPIStatusFlag;

#define	mpiStatusMaskBIT(flag)	(0x1 << (flag))

typedef enum {
	MPIStatusMaskNONE	= 0x0,


	MPIStatusMaskMOTOR	= MPIStatusMaskNONE,							/* 0x00000001 */

	MPIStatusMaskALL	= mpiStatusMaskBIT(MPIStatusFlagLAST) - 1			/* 0x00000001 */
} MPIStatusMask;

typedef enum MPIActionSource
{
	MPIActionSourceINVALID = -1,

	MPIActionSourceUSER,
	MPIActionSourceCONTROLLER,

	MPIActionSourceLAST,
	MPIActionSourceFIRST = MPIActionSourceINVALID + 1
} MPIActionSource;

typedef struct MPIStatus {
	MPIState		state;
	MPIAction		action;
	MPIEventMask	eventMask;

	MPI_BOOL		settled;
	MPI_BOOL		atTarget;

	MPIActionSource	actionSource;

	MPIStatusMask	statusMask;
} MPIStatus;

/* MPITrajectory */
typedef struct MPITrajectory {
	double	velocity;
	double	acceleration;
	double	deceleration;
	double	jerkPercent;
	double	accelerationJerk;
	double	decelerationJerk;
} MPITrajectory;

typedef struct _MPIPackAlignmentTest
{
	char   c;
	double d;
} _MPIPackAlignmentTest;

#define _MPI_PACK_ALIGNMENT_TEST (sizeof(_MPIPackAlignmentTest))
#define _MPI_INIT_CHECK_METHOD_VERSION (2)

#if defined(__cplusplus)
}
#endif

#endif
