/* path.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_PATH_H)
#define	_PATH_H

/* Path handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIPath *MPIPath;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "motion.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* Basic Path Element Types */

typedef enum {
	MPIPathElementTypeINVALID = -1,

	MPIPathElementTypeARC, /* only 2D */
	MPIPathElementTypeARC_CENTER, /* only 2D */
	MPIPathElementTypeARC_END_POINT, /* both 2D and 3D */
	MPIPathElementTypeHELIX, /* not currently supported */
	MPIPathElementTypeIO,  /* not currently supported */
	MPIPathElementTypeLINE,  /* both 2D and 3D */

	MPIPathElementTypeLAST,
	MPIPathElementTypeFIRST	= MPIPathElementTypeINVALID + 1,
	MPIPathElementTypeMASK	= 0xFF
} MPIPathElementType;

#define	mpiPathElementTYPE(type)	((type) & MPIPathElementTypeMASK)

/* Path Element Attributes */
typedef enum {
	MPIPathElementAttrINVALID	= -1,
	MPIPathElementAttrLAST		= 32,

	MPIPathElementAttrRELATIVE	= MPIPathElementAttrLAST - 1,
	MPIPathElementAttrID		= MPIPathElementAttrLAST - 2,
	MPIPathElementAttrVELOCITY	= MPIPathElementAttrLAST - 3,
	MPIPathElementAttrACCEL		= MPIPathElementAttrLAST - 4,
	MPIPathElementAttrTIMESLICE = MPIPathElementAttrLAST - 5,

	MPIPathElementAttrFIRST = MPIPathElementAttrLAST - 5,
	MPIPathElementAttrCOUNT = MPIPathElementAttrLAST - MPIPathElementAttrFIRST
} MPIPathAttr;

#define	mpiPathElementAttrMaskBIT(attr)	(0x1 << (attr))

typedef	enum {
	MPIPathElementAttrMaskRELATIVE	= mpiPathElementAttrMaskBIT(MPIPathElementAttrRELATIVE),
	MPIPathElementAttrMaskID		= mpiPathElementAttrMaskBIT(MPIPathElementAttrID),
	MPIPathElementAttrMaskVELOCITY	= mpiPathElementAttrMaskBIT(MPIPathElementAttrVELOCITY),
	MPIPathElementAttrMaskACCEL		= mpiPathElementAttrMaskBIT(MPIPathElementAttrACCEL),
	MPIPathElementAttrMaskTIMESLICE	= mpiPathElementAttrMaskBIT(MPIPathElementAttrTIMESLICE),

	MPIPathElementAttrMaskALL	= -1 << MPIPathElementAttrFIRST
} MPIPathElementAttrMask;

#define mpiPathElementATTR(type,attr)	((type) |= mpiPathElementAttrMaskBIT(attr))

typedef struct MPIPathElementAttributes {
	long	id;					/* MPIPathAttrID			*/
	double	velocity;			/* MPIPathAttrVELOCITY		*/
	double	acceleration;		/* MPIPathAttrACCELERATION	*/
	double	timeSlice;			/* MPIPathAttrTIMESLICE		*/
} MPIPathElementAttributes;

#define	MPIPathPointPOINTS_MAX		(5000)
#define	MPIPathPointDIMENSION_MAX	(16)

typedef struct MPIPathPoint {
	double	position[MPIPathPointDIMENSION_MAX];
} MPIPathPoint;

#define	MPIPathPointX(point)	((point).position[0])
#define	MPIPathPointY(point)	((point).position[1])
#define	MPIPathPointZ(point)	((point).position[2])

/* MPIPathParams structures for MPIPathType */

typedef enum {
	MPIPathDirectionCW	= -1,
	MPIPathDirectionCCW	= 1
} MPIPathDirection;

typedef struct MPIPathArc {
	struct {
		double	start;
		double	included;
	} angle;
	double	radius;
} MPIPathArc;

typedef struct MPIPathArcCenter {
	MPIPathPoint	center;
	double			angle;
} MPIPathArcCenter;

typedef struct MPIPathArcEndPoint {
	MPIPathPoint		center;
	MPIPathPoint		endPoint;
	MPIPathDirection	direction;
} MPIPathArcEndPoint;

typedef struct MPIPathLine {
	MPIPathPoint	point;
} MPIPathLine;

typedef	struct MPIPathElement {
	MPIPathElementType			type;
	long						blending;
	union {
		MPIPathArc				arc;
		MPIPathArcCenter		arcCenter;
		MPIPathArcEndPoint		arcEndPoint;
		MPIPathLine				line;
	} params;

	MPIPathElementAttributes	attributes;
} MPIPathElement;

typedef struct MPIPathParams {
	long			dimension;
	MPIPathPoint	start;
	double			velocity;
	double			acceleration;
	double			deceleration;
	MPIMotionType	interpolation;
	double			timeSlice;
	double			conversion[MPIPathPointDIMENSION_MAX][MPIPathPointDIMENSION_MAX];
} MPIPathParams;

typedef enum {
	MPIPathMessageFIRST = mpiMessageID(MPIModuleIdPATH, 0),

	MPIPathMessagePATH_INVALID,
	MPIPathMessageILLEGAL_DIMENSION,
	MPIPathMessageILLEGAL_ELEMENT,
	MPIPathMessageARC_ILLEGAL_DIMENSION,
	MPIPathMessageHELIX_ILLEGAL_DIMENSION,
	MPIPathMessageILLEGAL_RADIUS,
	MPIPathMessagePATH_TOO_LONG,
	MPIPathMessageILLEGAL_VELOCITY,
	MPIPathMessageILLEGAL_ACCELERATION ,
	MPIPathMessageILLEGAL_TIMESLICE ,
	MPIPathMessageINVALID_BLENDING,

	MPIPathMessageLAST
} MPIPathMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIPath MPI_DECL2
	mpiPathCreate();

MPI_DECL1 long MPI_DECL2
	mpiPathDelete(MPIPath path);

MPI_DECL1 long MPI_DECL2
	mpiPathValidate(MPIPath path);

/* Path */
MPI_DECL1 long MPI_DECL2
	mpiPathAppend(MPIPath			path,
				  MPIPathElement	*element);

MPI_DECL1 long MPI_DECL2
	mpiPathParamsGet(MPIPath		path,
					 MPIPathParams	*params,
					 void			*external);

MPI_DECL1 long MPI_DECL2
	mpiPathParamsSet(MPIPath		path,
					 MPIPathParams	*params,
					 void			*external);

MPI_DECL1 long MPI_DECL2
	mpiPathMotionParamsGenerate(MPIPath			path,
								MPIMotionParams	*params);


#if defined(__cplusplus)
}
#endif

#endif
