    /* probe.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_PROBE_H)
#define	_PROBE_H

/* Probe handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIProbe *MPIProbe;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "motor.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define MPIProbeNOT_MAPPED		(-1)
#define MPIMaxProbeRegisters	(16)
#define MPIMaxProbeRegisterSet  (8)

typedef enum MPIProbeSource {
	MPIProbeSourceINVALID =		-1,

	MPIProbeSourceHOME,
	MPIProbeSourceINDEX,
	MPIProbeSourceLIMIT_HW_NEG,
	MPIProbeSourceLIMIT_HW_POS,
	MPIProbeSourceINDEX_SECONDARY,
	MPIProbeSourceMOTOR_IO_0,
	MPIProbeSourceMOTOR_IO_1,
	MPIProbeSourceMOTOR_IO_2,
	MPIProbeSourceMOTOR_IO_3,
	MPIProbeSourceMOTOR_IO_4,
	MPIProbeSourceMOTOR_IO_5,
	MPIProbeSourceMOTOR_IO_6,
	MPIProbeSourceMOTOR_IO_7,
	MPIProbeSourceMOTOR_IO_8,
	MPIProbeSourceMOTOR_IO_9,
	MPIProbeSourceMOTOR_IO_10,
	MPIProbeSourceMOTOR_IO_11,
	MPIProbeSourceMOTOR_IO_12,
	MPIProbeSourceMOTOR_IO_13,
	MPIProbeSourceMOTOR_IO_14,
	MPIProbeSourceMOTOR_IO_15,

	MPIProbeSourceLAST,
	MPIProbeSourceFIRST = MPIProbeSourceINVALID + 1,

	MPIProbeSourceCOUNT = MPIProbeSourceLAST
} MPIProbeSource;

typedef enum  MPIProbeData {
	MPIProbeDataINVALID = -1,
	
	MPIProbeDataPOSITION_PRIMARY,
	MPIProbeDataPOSITION_SECONDARY,
	MPIProbeDataTIME,

	MPIProbeDataLAST,
	MPIProbeDataFIRST = MPIProbeDataINVALID +1 
} MPIProbeData;

/* Probe Config */
typedef struct MPIProbeConfig {
	MPI_BOOL				enable;				  /* TRUE/FALSE */
	MPIProbeSource			source;
	MPIProbeData			data;
	MPI_BOOL				inputFilter;		  /* TRUE/FALSE */
} MPIProbeConfig;

typedef struct MPIProbeStatus {
	long	maxRegisters;
	long	index;
	long	io;
	long	regs[MPIMaxProbeRegisters];
} MPIProbeStatus;

typedef enum {
	MPIProbeMessageFIRST = mpiMessageID(MPIModuleIdPROBE, 0),

	MPIProbeMessageNODE_INVALID,
	MPIProbeMessagePROBE_TYPE_INVALID,
	MPIProbeMessagePROBE_INVALID,

	MPIProbeMessageLAST
} MPIProbeMessage;

typedef enum MPIProbeType {
	MPIProbeTypeMOTOR,
	MPIProbeTypeIO
} MPIProbeType;

typedef union MPIProbeNumber {
	long		motor;
	long		node;
} MPIProbeNumber;

typedef struct MPIProbeParams {
	MPIProbeType			type;
	MPIProbeNumber			number;
	long					probeIndex;
} MPIProbeParams;

/* Create/Delete/Validate */
MPI_DECL1 MPIProbe MPI_DECL2
	mpiProbeCreate(MPIControl			control,
					 MPIProbeParams		*params);

MPI_DECL1 long MPI_DECL2
	mpiProbeDelete(MPIProbe	probe);

MPI_DECL1 long MPI_DECL2
	mpiProbeValidate(MPIProbe	probe);

/* Probe */
MPI_DECL1 long MPI_DECL2
	mpiProbeConfigGet(MPIProbe			probe,
						MPIProbeConfig	*config,
						void			*external);

MPI_DECL1 long MPI_DECL2
	mpiProbeConfigSet(MPIProbe			probe,
						MPIProbeConfig	*config,
						void			*external);
MPI_DECL1 MPIControl MPI_DECL2
	mpiProbeControl(MPIProbe probe);

MPI_DECL1 long MPI_DECL2
	mpiProbeStatus(MPIProbe		probe,
				 MPIProbeStatus	*status,
				 void			*external);

MPI_DECL1 long MPI_DECL2
	mpiProbeParams(MPIProbe			probe,
				   MPIProbeParams 	*params);

#if defined(__cplusplus)
}
#endif

#endif
