/* remote.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_REMOTE_H)
#define	_REMOTE_H

#include "meidef.h"
#include "platform.h"

#if defined(__cplusplus)
extern "C" {
#endif

struct __MPIString;
typedef struct __MPIString* _MPIString;

#define	MEIRemotePORT	(3300)

typedef enum {
	MEIRemoteTypeINVALID = -1,

	MEIRemoteTypeQUERY,
	MEIRemoteTypeREPLY,

	MEIRemoteTypeLAST,
	MEIRemoteTypeFIRST = MEIRemoteTypeINVALID + 1
} MEIRemoteType;

typedef enum {
	MEIRemoteMethodINVALID = -1,

	MEIRemoteMethodBOARD_TYPE,
	MEIRemoteMethodBOARD_INFO_GET,
	MEIRemoteMethodBOARD_INFO_SET,

	MEIRemoteMethodFLASH_MEMORY_GET,
	MEIRemoteMethodFLASH_MEMORY_SET,

	MEIRemoteMethodINTERRUPT_ENABLE,
	MEIRemoteMethodINTERRUPT_WAIT,
	MEIRemoteMethodINTERRUPT_WAKE,

	MEIRemoteMethodMEMORY,

	MEIRemoteMethodMEMORY_GET,
	MEIRemoteMethodMEMORY_SET,

	MEIRemoteMethodMEMORY_GET64,
	MEIRemoteMethodMEMORY_SET64,

	MEIRemoteMethodMEM_WINDOW_CHECK,
	MEIRemoteMethodMEM_WINDOW_GET,
	MEIRemoteMethodMEM_WINDOW_SET,

	MEIRemoteMethodPLATFORM_INFO,

	MEIRemoteMethodPORT_IN_CHAR,
	MEIRemoteMethodPORT_OUT_CHAR,

	MEIRemoteMethodOBJECT_LOCK_GIVE,
	MEIRemoteMethodOBJECT_LOCK_TAKE,

	MEIRemoteMethodLINK_LOCK_GIVE,
	MEIRemoteMethodLINK_LOCK_TAKE,

	MEIRemoteMethodRESET,

	MEIRemoteMethodSYSTEM_LOCK,
	MEIRemoteMethodSYSTEM_UNLOCK,

	MEIRemoteMethodEXMP_TEMP_INIT,
	MEIRemoteMethodEXMP_TEMP_GET,

	MEIRemoteMethodFLASH_IMAGE_SET,
	MEIRemoteMethodSQNODE_FLASH,

	MEIRemoteMethodCOMPATIBILITY,

	MEIRemoteMethodCONTROL_VERSION_TEXT,

	MEIRemoteMethodFLASH_SECTOR_ERASE,

	MEIRemoteMethodMOTION_STATUS,
	
	MEIRemoteMethodSQNODE_DRIVE_MONITOR,
	
	MEIRemoteMethodAXIS_ACTUAL_POSITION_GET,
	MEIRemoteMethodAXIS_POSITION_ERROR,
	
	MEIRemoteMethodMOTOR_AMP_ENABLE_GET,
	
	MEIRemoteMethodUSER_RPC,
	MEIRemoteMethodCONTROL_STATUS_CACHE_GET,

	MEIRemoteMethodLAST,
	MEIRemoteMethodFIRST = MEIRemoteMethodINVALID + 1
} MEIRemoteMethod;

typedef struct MEIRemoteMethodBoardInfoGet {
	MEIPlatformBoardType	boardType;
	void					*boardInfo;
} MEIRemoteMethodBoardInfoGet;

typedef MEIRemoteMethodBoardInfoGet MEIRemoteMethodBoardInfoSet;

typedef long* MEIRemoteMethodExmpTempGet;

typedef struct MEIRemoteMethodPortInChar {
	unsigned short	port;
	unsigned char	*value;
} MEIRemoteMethodPortInChar;

typedef struct MEIRemoteMethodPortOutChar {
	unsigned short	port;
	unsigned char	value;
} MEIRemoteMethodPortOutChar;

typedef struct MEIRemoteMethodInterruptWait {
	long	*interrupted;
	MPIWait	timeout;
} MEIRemoteMethodInterruptWait;

typedef struct MEIRemoteMethodMemory {
	void	**firmware;
	void	**external;
} MEIRemoteMethodMemory;

typedef struct MEIRemoteMethodMemoryGet {
	void	   *dst;
	const void *src;
	long	   count;
} MEIRemoteMethodMemoryGet;

typedef MEIRemoteMethodMemoryGet MEIRemoteMethodMemorySet;

typedef struct MEIRemoteMethodMemWindowCheck {
	const void *addr;
	long	   count;
} MEIRemoteMethodMemWindowCheck;

typedef struct MEIRemoteMethodObjectLockGive {
	MPIModuleId	moduleId;
	long		number;
} MEIRemoteMethodObjectLockGive;

typedef struct MEIRemoteMethodObjectLockTake {
	MPIModuleId	moduleId;
	long		number;
	MPIWait		timeout;
} MEIRemoteMethodObjectLockTake;

typedef struct MEIRemoteMethodLockTake {
	MPIWait		timeout;
} MEIRemoteMethodLinkLockTake;

typedef struct MEIRemoteMethodsSystemLock {
	long		*locksTaken;
	MPIWait		timeout;
} MEIRemoteMethodsSystemLock;

typedef struct MEIRemoteMethodsSystemUnlock {
	long		lockCount;
} MEIRemoteMethodsSystemUnlock;

typedef struct MEIRemoteMethodFlashImageSet {
	MEIFlashFileType	type;
	long				size;
	unsigned char*		image;
} MEIRemoteMethodFlashImageSet;

typedef struct MEIRemoteMethodSqNodeFlash {
	long					size;
	long					nodeNumber;
	long*					image;
	MEISqNodeCallback		callback;
} MEIRemoteMethodSqNodeFlash;

typedef struct MEIRemoteMethodPlatformInfo {
	void	*data;
} MEIRemoteMethodPlatformInfo;

typedef struct MEIRemoteMethodCompatiblity {
	int		remoteMethodChecksum;	/* method compatibility */
	int		mpiInterfaceVersion;	/* MPI interface compatibility */
} MEIRemoteMethodCompatiblity;

typedef struct MEIRemoteMethodControlVersionText {
	_MPIString text;
} MEIRemoteMethodControlVersionText;

typedef struct MEIRemoteMethodFlashSectorErase {
	long		sector;
	MPI_BOOL	erase;
	MPI_BOOL	verify;
} MEIRemoteMethodFlashSectorErase;

typedef struct MEIRemoteMethodMotionStatus {
	long motionNumber;
	long axisIndexList[MEIXmpMAX_COORD_AXES];
	MPIStatus *status;
} MEIRemoteMethodMotionStatus;

typedef struct MEIRemoteMethodSqNodeDriveMonitor {
	long nodeNumber;
	long driveIndex;
	MEISqNodeMonitorValue *monitor;
} MEIRemoteMethodSqNodeDriveMonitor;

typedef struct MEIRemoteMethodAxisPosition {
	long axisNumber;
	double *position;
} MEIRemoteMethodAxisPosition;

typedef struct MEIRemoteMethodMotorAmpEnable {
	long motorNumber;
	MPI_BOOL *ampEnable;
} MEIRemoteMethodMotorAmpEnable;

typedef struct MEIRemoteMethodControlStatus {
	void*	cache;
	long	cacheSize;
} MEIRemoteMethodControlStatus;

typedef union {
	MEIPlatformBoardType*				boardType;

	MEIRemoteMethodBoardInfoGet			boardInfoGet;
	MEIRemoteMethodBoardInfoSet			boardInfoSet;
	MEIRemoteMethodExmpTempGet			temperature;
	union {
		long							enable;
		MEIRemoteMethodInterruptWait	wait;
	} interruptArgs;
	MEIRemoteMethodMemory				memory;
	MEIRemoteMethodMemoryGet			memoryGet;
	MEIRemoteMethodMemorySet			memorySet;
	MEIRemoteMethodMemWindowCheck		memWindowCheck;
	MEIRemoteMethodPlatformInfo			platformInfo;
	MEIRemoteMethodPortInChar			portIn;
	MEIRemoteMethodPortOutChar			portOut;
	MEIRemoteMethodObjectLockGive		objectLockGive;
	MEIRemoteMethodObjectLockTake		objectLockTake;
	MEIRemoteMethodLinkLockTake			linkLockTake;
	MEIRemoteMethodsSystemLock			systemLock;
	MEIRemoteMethodsSystemUnlock		systemUnlock;
	MEIRemoteMethodFlashImageSet		fileImage;
	MEIRemoteMethodSqNodeFlash			sqNodeFlash;
	MEIRemoteMethodCompatiblity			compatibility;
	MEIRemoteMethodControlVersionText	controlVersionText;
	MEIRemoteMethodFlashSectorErase		flashSectorErase;
	MEIRemoteMethodMotionStatus			motionStatusGet;
	MEIRemoteMethodSqNodeDriveMonitor	sqNodeDriveMonitor;
	MEIRemoteMethodAxisPosition			axisPosition;
	MEIRemoteMethodMotorAmpEnable		motorAmpEnable;
	MPIControlUserRpcData				userRpc;
	MEIRemoteMethodControlStatus		controlStatus;

} MEIRemoteMethodArgs;

typedef struct MEIRemoteHeader {
	unsigned long	size;	/* bytes */
	unsigned long	sequence;

	MEIRemoteType	type;
	union {
		struct {
			MEIRemoteMethod		method;
			MEIRemoteMethodArgs	args;
		} query;
		struct {
			long	returnValue;
			union {
				MEIPlatformBoardType	boardType;	/* meiPlatformBoardType() */
				unsigned char portValue;
				MPI_BOOL	  interrupted;	/* mpiControlInterruptWait() */
				struct {
					void	*firmware;
					void	*external;
				} memory;	/* mpiControlMemory() */
				long temperature;
				long sysLocksTaken;
				long lenVersion;
				long additionalDataSize;
			} output;
		} reply;
	} asa;

	unsigned long	memory;	/* mpiControlMemory[GS]et(count == 4) */
} MEIRemoteHeader;

#if defined(__cplusplus)
}
#endif

#endif
