/* sqNode.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_SQNODE_H)
#define	_SQNODE_H

/* SqNode handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIBlock *MEISqNode; 
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "drivemap.h"
#include "xmp.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define MEISqNodeID_CHAR_MAX					(30)	
#define MEISqNodeDriveParamMAX_STRING_LENGTH	MEIDriveMapParamMAX_STRING_LENGTH

#define MEISqNodeMaxMOTORS						(MEIXmpMotorsPerBlock)   
#define MEISqNodeMaxFEEDBACK_SECONDARY			(MEISqNodeMaxMOTORS)
#define MEISqNodeNOT_AVAILABLE					(-1)
	
#define MEISqNodeConfigControlLatencyMIN_LIMIT	(1000)
#define MEISqNodeConfigControlLatencyMAX_LIMIT	(1000)

typedef long (MPI_DEF2 *MEISqNodeCallback)(long percentage);

/* SqNode Commands */
typedef enum MEISqNodeChannel {
	MEISqNodeChannelINVALID = -1,

	MEISqNodeChannelDRIVE0, 
	MEISqNodeChannelDRIVE1,
	MEISqNodeChannelDRIVE2,
	MEISqNodeChannelDRIVE3,
	MEISqNodeChannelDRIVE4,
	MEISqNodeChannelDRIVE5,
	MEISqNodeChannelDRIVE6,
	MEISqNodeChannelDRIVE7,
	MEISqNodeChannelNODE,		  

	MEISqNodeChannelLAST,
	MEISqNodeChannelFIRST = MEISqNodeChannelINVALID + 1,

	MEISqNodeChannelDRIVE_FIRST = MEISqNodeChannelDRIVE0,
	MEISqNodeChannelDRIVE_LAST  = MEISqNodeChannelDRIVE7 + 1
} MEISqNodeChannel;

typedef enum MEISqNodeMemory {
	MEISqNodeMemoryINVALID = -1,

	MEISqNodeMemoryDATA,				/* node/drive processor RAM */
	MEISqNodeMemoryPROGRAM,				/* drive processor program memory */
	MEISqNodeMemoryIO,					/* drive I/O memory */
	MEISqNodeMemoryDRIVE,				/* direct command to drive */
	
	MEISqNodeMemoryLAST,
	MEISqNodeMemoryFIRST = MEISqNodeMemoryINVALID + 1
} MEISqNodeMemory;

typedef enum MEISqNodeDataSize {		/* read/write data width */
	MEISqNodeDataSizeINVALID = -1,

	MEISqNodeDataSize8BIT,
	MEISqNodeDataSize16BIT,
	MEISqNodeDataSize24BIT,
	MEISqNodeDataSize32BIT,

	MEISqNodeDataSizeLAST,
	MEISqNodeDataSizeFIRST = MEISqNodeDataSizeINVALID + 1
} MEISqNodeDataSize;

typedef enum MEISqNodeCmdType {
	MEISqNodeCmdTypeINVALID = -1,

	MEISqNodeCmdTypeREAD,
	MEISqNodeCmdTypeWRITE,

	MEISqNodeCmdTypeLAST,
	MEISqNodeCmdTypeFIRST = MEISqNodeCmdTypeINVALID + 1
} MEISqNodeCmdType;

typedef struct MEISqNodeCmdHeader {
	MEISqNodeChannel 	channel;		/* internal node destination */
	MEISqNodeMemory		memory;
	MEISqNodeDataSize	size;
	MEISqNodeCmdType	type;			/* read/write command */
} MEISqNodeCmdHeader;

typedef struct MEISqNodeCommand {
	MEISqNodeCmdHeader	header;
	unsigned long		address;		/* command registers */
	unsigned long		data;			/* command data */
} MEISqNodeCommand;

typedef struct MEISqNodeResponse {
	unsigned long		data;			/* response data */
} MEISqNodeResponse;

/* SqNode Config */
typedef struct MEISqNodeConfigTrigger {
	MPI_BOOL enable;
	MPI_BOOL invert;
} MEISqNodeConfigTrigger;

typedef struct MEISqNodeConfigPacketError {
	long faultLimit;					 /* 0 - 255 */
	long failLimit;						 /* 0 - 255 */
} MEISqNodeConfigPacketError;

typedef struct MEISqNodeConfigUserFault {
	long			*addr;				 /* firmware addr */
	unsigned long	mask;
	unsigned long	pattern;
} MEISqNodeConfigUserFault;

typedef struct MEISqNodeConfigIoAbort {
	MEISqNodeConfigTrigger  synqLost;	 /* communication error	 */
	MEISqNodeConfigTrigger	nodeDisable; /* external input		 */
	MEISqNodeConfigTrigger  powerFault;	 /* analog power failure */
	MPI_BOOL				userFault;	 /* TRUE = user fault causes ioAbort */
} MEISqNodeConfigIoAbort;

typedef struct MEISqNodeConfigAlarm {
	unsigned long	mask;				/* One bit per drive/motor. Triggered by   
										   the MEIMotorDedicatedInAMP_FAULT input. */ 
	MPI_BOOL		notCyclicEnable;	/* allow nodeAlarm to be asserted when the node
										   is not in cyclic mode */
	MPI_BOOL		ioAbortEnable;		/* allow ioAbort to assert nodeAlarm	*/
	MPI_BOOL		ioFaultEnable;		/* allow ioFault to assert nodeAlarm	*/
} MEISqNodeConfigAlarm;	

typedef struct MEISqNodeConfigControlLatency {
	double minimum;	/* uS  - if > 0, this will insure a minimum latency value */
	double maximum;	/* uS  - if > 0, an error will occur if this value is exceeded */
} MEISqNodeConfigControlLatency;

typedef struct MEISqNodeFeedbackSecondary {
	long motorIndex;
} MEISqNodeFeedbackSecondary;

typedef struct MEISqNodeConfig {
	char					   userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */
	MEISqNodeConfigAlarm	   nodeAlarm;
	MEISqNodeConfigIoAbort	   ioAbort;
	MEISqNodeConfigPacketError upStreamError;
	MEISqNodeConfigPacketError downStreamError;
	MEISqNodeConfigUserFault   userFault;
	
	MEISqNodeConfigControlLatency controlLatency;

	MEISqNodeFeedbackSecondary feedbackSecondary[MEISqNodeMaxFEEDBACK_SECONDARY]; 
} MEISqNodeConfig;

/* SqNode Download */
typedef struct MEISqNodeDownloadParams {
	char			  *filename;
	MEISqNodeChannel  channel;
	MEISqNodeCallback callback;
} MEISqNodeDownloadParams;

/* SqNode Information */
typedef enum MEISqNodeFpgaType {
	MEISqNodeFpgaTypeINVALID = -1,
	
	MEISqNodeFpgaTypeBOOT,
	MEISqNodeFpgaTypeRUN_TIME,

	MEISqNodeFpgaTypeLAST,
	MEISqNodeFpgaTypeFIRST = MEISqNodeFpgaTypeINVALID + 1
} MEISqNodeFpgaType;

	/*FPGA version type  */
typedef  enum MEISqNodeFPGAVersionType {
	MEISqNodeFPGAVersionTypeINVALID = -1,

	MEISqNodeFPGAVersionTypeDEFAULT,               /*Network will be in synqMode.*/
	MEISqNodeFPGAVersionTypeCOMPATIBLE,            /*Network will be in synqMode.*/
	MEISqNodeFPGAVersionTypeLAB,                   /* Network will be in SynqMode */
	MEISqNodeFPGAVersionTypeNOT_COMPATIBLE,        /* Network Will be Asynq. */
	MEISqNodeFPGAVersionTypeBOOT,                  /* Network Will be Asynq. */
	MEISqNODEFPGAVersionTypeMAC_NOT_COMPATIBLE,      /*Network will be Asynq */

	MEISqNodeFPGAVersionTypeLAST,
	MEISqNodeFPGAVersionTypeFIRST = MEISqNodeFPGAVersionTypeINVALID +1
}MEISqNodeFPGAVersionType;


#define MEISqNodeManufacturerDATA_CHAR_MAX	(0x10)

typedef struct MEISqNodeInfoId {
	unsigned long nodeType;	  /* product/mfg code   */
	char		  *nodeName;  /* product/mfg string */
	unsigned long option;	  /* product option code*/
	unsigned long switchId;	  /* switch id          */
	unsigned long unique;	  /* unique id code     */

	MPI_BOOL	  exactMatch; /* TRUE/FALSE		    */

	char		  serialNumber[MEISqNodeID_CHAR_MAX];
	char		  modelNumber[MEISqNodeID_CHAR_MAX];
	char		  manufacturerData[MEISqNodeManufacturerDATA_CHAR_MAX];
} MEISqNodeInfoId;

typedef struct MEISqNodeInfoFpga {
	MEISqNodeFpgaType type;		
	unsigned long	  vendorDevice;
	unsigned long	  version;
	unsigned long     branchVersion;
	MPI_BOOL		  defaultVersion; /* TRUE/FALSE */
} MEISqNodeInfoFpga;

typedef struct MEISqNodeInfoNetwork {
	long number;
	long inPorts;
	long outPorts;
} MEISqNodeInfoNetwork;

typedef struct MEISqNodeInfoIo {
	long digitalInCount;
	long digitalOutCount;
	long analogInCount;
	long analogOutCount;
	long segmentCount;
	long maxWait;
} MEISqNodeInfoIo;

typedef struct MEISqNodeInfo {
	long				 motorCount;
	long				 driveCount;
	long				 motorOffset;
	long				 feedbackSecondaryCount;
	MEISqNodeInfoId		 id;
	MEISqNodeInfoFpga	 fpga;
	MEISqNodeInfoNetwork network;
	MEISqNodeInfoIo      io;
} MEISqNodeInfo;


#define MEISqNodeUserDATA_CHAR_MAX	(0x10)

typedef struct MEISqNodeUserData {
	char		data[MEISqNodeUserDATA_CHAR_MAX];	
} MEISqNodeUserData;

#define MEISqNodeSegmentUserDATA_CHAR_MAX	(0x10)

typedef struct MEISqNodeSegmentUserData {
	char		data[MEISqNodeSegmentUserDATA_CHAR_MAX];	
} MEISqNodeSegmentUserData;

typedef struct MEISqNodeDriveInfo {
	char	firmwareVersion[MEISqNodeDriveParamMAX_STRING_LENGTH];
} MEISqNodeDriveInfo;

/* SqNode Status */
#define MEISqNodeSTATUS_NOT_AVAILLABLE (-1)

typedef struct MEISqNodeStatusPacketError {
	long rate;			
	long count;
} MEISqNodeStatusPacketError;

typedef struct MEISqNodeStatusCrcError {
	long port[MEINetworkPortLAST];
} MEISqNodeStatusCrcError;

typedef struct MEISqNodeStatusIoFaults {
	MPI_BOOL digitalIn;
	MPI_BOOL digitalOut;
	MPI_BOOL analogIn;
	MPI_BOOL analogOut;
} MEISqNodeStatusIoFaults;

typedef struct MEISqNodeStatus {
	MEISqNodeStatusPacketError	upStreamError;
	MEISqNodeStatusPacketError	downStreamError;
	MEISqNodeStatusCrcError		crcError;
	MPIEventMask				eventMask;
	MEISqNodeStatusIoFaults		ioFaults;
} MEISqNodeStatus;

/* SqNode Segments */
#define MEISqNodeSegmentInfoSERIAL_NUMBER_LENGTH	0x20
#define MEISqNodeSegmentInfoMODEL_NAME_LENGTH		0x20
#define MEISqNodeSegmentInfoMANUFACTURER_LENGTH		0x10

typedef struct MEISqNodeSegmentInfo {
	long id;
	long option;
	char serialNumber[MEISqNodeSegmentInfoSERIAL_NUMBER_LENGTH];
	char modelName[MEISqNodeSegmentInfoMODEL_NAME_LENGTH];
	long digitalInCount;
	long digitalOutCount;
	long analogInCount;
	long analogOutCount;
	long version;
	long paramCount;
	long memoryCount;
	char manufacturerData[MEISqNodeManufacturerDATA_CHAR_MAX];
} MEISqNodeSegmentInfo;

/* SqNode Drive Monitor */
typedef union {
	long index;         /* the values for these parameters are drive specific	*/
	long address;		/* and can be found in the appropriate drive modules	*/
} MEISqNodeDriveMonitorData;

typedef enum MEISqNodeDriveMonitorDataType {
	MEISqNodeDriveMonitorDataTypeINVALID = -1,

	MEISqNodeDriveMonitorDataTypeINDEX,
	MEISqNodeDriveMonitorDataTypeADDRESS,
	MEISqNodeDriveMonitorDataTypeFIXED_CONFIG,
	
	MEISqNodeDriveMonitorDataTypeLAST,
	MEISqNodeDriveMonitorDataTypeFIRST = MEISqNodeDriveMonitorDataTypeINVALID +1
} MEISqNodeDriveMonitorDataType;


typedef struct MEISqNodeDriveMonitor {
	MEISqNodeDriveMonitorDataType type;
	MEISqNodeDriveMonitorData	  data;
} MEISqNodeDriveMonitor;

typedef struct MEISqNodeDriveMonitorConfig {
	MEISqNodeDriveMonitor monitorA;
	MEISqNodeDriveMonitor monitorB;
	MEISqNodeDriveMonitor monitorC;
}MEISqNodeDriveMonitorConfig;

typedef enum MEISqNodeMonitorValueIndex {
	MEISqNodeMonitorValueIndexINVALID = -1,

	MEISqNodeMonitorValueIndexA,
	MEISqNodeMonitorValueIndexB,
	MEISqNodeMonitorValueIndexC,

	MEISqNodeMonitorValueIndexLAST,
	MEISqNodeMonitorValueIndexFIRST = MEISqNodeMonitorValueIndexINVALID +1
} MEISqNodeMonitorValueIndex;

typedef struct MEISqNodeMonitorValue {
	long count;
	long monitor[MEISqNodeMonitorValueIndexLAST];
} MEISqNodeMonitorValue;

typedef struct MEISqNodeMonitorLocation {
	long			*addr;
	unsigned long	mask;
	unsigned long	rightShift;
} MEISqNodeMonitorLocation;

typedef struct MEISqNodeMonitorConfigInfo{
	MEISqNodeDriveMonitorDataType type;
	long						  value;
	const char					  *description;
} MEISqNodeMonitorConfigInfo;

typedef struct MEISqNodeDriveMonitorInfo {
	long								monitorCount;
	MEISqNodeMonitorLocation			location[MEISqNodeMonitorValueIndexLAST];
	long								configCount;
	const MEISqNodeMonitorConfigInfo	*configInfo;		/* array of configuration information of size
														   configCount, this info is drive specific */
}MEISqNodeDriveMonitorInfo;



typedef enum {
	MEISqNodeDriveParamCallbackTypeCHANGED,
	MEISqNodeDriveParamCallbackTypeSET_FAILED,
	MEISqNodeDriveParamCallbackTypeSKIPPING_READ_ONLY
} MEISqNodeDriveParamCallbackType;

typedef void (MPI_DEF2 *MEISqNodeDriveParamCallback)(MEISqNodeDriveParamCallbackType type,
													 const char *name,
													 long number,
													 const char *value);

typedef enum {
	MEISqNodeMessageFIRST = mpiMessageID(MEIModuleIdSQNODE, 0),

	MEISqNodeMessageINVALID,
	MEISqNodeMessageNODE_INVALID,
	MEISqNodeMessageSTATE_ERROR,
	MEISqNodeMessageCONFIG_NETWORK_MISMATCH,
	MEISqNodeMessageMAP_CONFIG_MISMATCH,
	MEISqNodeMessageNOT_IN_CONFIG_FILE,
	MEISqNodeMessageCONFIG_FILE_FORMAT_INVALID,

	MEISqNodeMessageRESPONSE_TIMEOUT,
	MEISqNodeMessageREADY_TIMEOUT,
	MEISqNodeMessageSRVC_ERROR,
	MEISqNodeMessageSRVC_UNSUPPORTED,
	MEISqNodeMessageSRVC_CHANNEL_INVALID,

	MEISqNodeMessageCMD_NOT_SUPPORTED,
	MEISqNodeMessageDISCOVERY_FAILURE,
	MEISqNodeMessageDISPATCH_ERROR,
	MEISqNodeMessageINIT_FAILURE,
	MEISqNodeMessageINTERFACE_ERROR1,
	MEISqNodeMessageFILE_NODE_MISMATCH,
	MEISqNodeMessageFILE_INVALID,
	MEISqNodeMessageINVALID_HEADER,
	MEISqNodeMessageDOWNLOAD_FAIL,
	MEISqNodeMessageVERIFY_FAIL,
	MEISqNodeMessageDOWNLOAD_NOT_SUPPORTED,
	MEISqNodeMessageVERIFY_NOT_SUPPORTED,
	MEISqNodeMessageBOOT_ROM_INVALID,
	MEISqNodeMessageINVALID_TABLE,
	MEISqNodeMessageINVALID_STR_LEN,
	MEISqNodeMessageFEEDBACK_MAP_INVALID,
	MEISqNodeMessageNODE_FAILURE,
	MEISqNodeMessageEXCEEDED_MAXIMUM_SYNQNET_PACKET_LIMIT,

	MEISqNodeMessageIO_MODULE_INCOMPATIBILITY,
	MEISqNodeMessageIO_MODULE_EEPROM_NOT_PROGRAMMED,
	MEISqNodeMessageIO_MODULE_COUNT_EXCEEDED,
	MEISqNodeMessageIO_MODULE_LENGTH_CHECK_FAILED,
	MEISqNodeMessageIO_MODULE_3_3V_BUS_CURRENT_EXCEEDED,
	MEISqNodeMessageIO_MODULE_24V_BUS_CURRENT_EXCEEDED,

	MEISqNodeMessageIO_SLICE_INITIALIZATION_FAULT,
	MEISqNodeMessageIO_SLICE_INITIALIZATION_FAULT_TOO_MANY_SLICES,
	MEISqNodeMessageIO_SLICE_INITIALIZATION_FAULT_VENDOR_MISMATCH,
	MEISqNodeMessageIO_SLICE_INITIALIZATION_TIMEOUT,
	MEISqNodeMessageIO_SLICE_TOPOLOGY_MISMATCH,
	MEISqNodeMessageIO_SLICE_SERVICE_RECEIVE_ERROR,
	MEISqNodeMessageIO_SLICE_SERVICE_TOO_MANY_CHAR,
	MEISqNodeMessageIO_SLICE_SERVICE_BUS_ERROR_CODE,
	MEISqNodeMessageIO_SLICE_SERVICE_UNKNOWN_FAULT_CODE,
	MEISqNodeMessageIO_SLICE_SERVICE_RESOURCE_UNAVAILABLE,
	MEISqNodeMessageIO_SLICE_SERVICE_NOT_SUPPORTED,
	MEISqNodeMessageIO_SLICE_SERVICE_INVALID_ATTRIBUTE_VALUE,
	MEISqNodeMessageIO_SLICE_SERVICE_ALREADY_IN_MODE,
	MEISqNodeMessageIO_SLICE_SERVICE_STATE_CONFLICT,
	MEISqNodeMessageIO_SLICE_SERVICE_ATTRIBUTE_NOT_SETTABLE,
	MEISqNodeMessageIO_SLICE_SERVICE_NOT_ENOUGH_DATA,
	MEISqNodeMessageIO_SLICE_SERVICE_ATTRIBUTE_NOT_SUPPORTED,
	MEISqNodeMessageIO_SLICE_SERVICE_TOO_MUCH_DATA,
	MEISqNodeMessageIO_SLICE_SERVICE_OBJECT_DOES_NOT_EXIST,
	MEISqNodeMessageIO_SLICE_SERVICE_INVALID_PARAMETER,
	MEISqNodeMessageIO_SLICE_SERVICE_STORE_OPERATION_FAILURE,
	MEISqNodeMessageIO_SLICE_SERVICE_UNKNOWN_ERROR_CODE,
	MEISqNodeMessageIO_SLICE_SERVICE_TIMEOUT,
	MEISqNodeMessageIO_SLICE_SERVICE_RESPONSE_FORMAT,
	MEISqNodeMessageIO_SLICE_EEPROM_FORMAT,
	MEISqNodeMessageIO_SLICE_TOO_MUCH_IO,

	MEISqNodeMessageBOOT_FILE_NOT_FOUND,
	MEISqNodeMessagePARAM_READ_ONLY,
	MEISqNodeMessagePARAM_LOCKED,
	MEISqNodeMessageMONITOR_INDEX,  
	MEISqNodeMessageMONITOR_ADDRESS,
	MEISqNodeMessageMONITOR_NA,

	MEISqNodeMessageCOMMUTATION_INIT_FAILURE,
	MEISqNodeMessagePOSITION_CLEAR_FAILURE,
	MEISqNodeMessageNODE_EEPROM_SET,
	MEISqNodeMessageDRIVE_FIRMWARE_FLASH_EARSE_FAILURE,
	MEISqNodeMessageDRIVE_MULTITURN_POSITION_READ_FAILURE,
	MEISqNodeMessagePOSITION_FEEDBACK_OFFSET_UNINITIALIZED,

	MEISqNodeMessage_FIRMWARE_FILE_CRC_ERROR,
	MEISqNodeMessage_FIRMWARE_DOWNLOAD_WARNING,

	MEISqNodeMessageUART_READ_FAIL,
	MEISqNodeMessageUART_READ_TIMEOUT,
	MEISqNodeMessageLAST
} MEISqNodeMessage;


#define MEISqNodeFILENAME_MAX (24)

typedef struct MEISqNodeFileName{
	char fileName[MEISqNodeFILENAME_MAX];
}MEISqNodeFileName;

/* Create/Delete/Validate */
MPI_DECL1 MEISqNode MPI_DECL2
	meiSqNodeCreate(MPIControl	control,
					long		number);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDelete(MEISqNode node);

MPI_DECL1 long MPI_DECL2
	meiSqNodeValidate(MEISqNode node);

/* SqNode */
MPI_DECL1 long MPI_DECL2
	meiSqNodeCommand(MEISqNode		   node,
					 MEISqNodeCommand  *command,
					 MEISqNodeResponse *response);

MPI_DECL1 long MPI_DECL2
	meiSqNodeConfigGet(MEISqNode	   node,
					   MEISqNodeConfig *config);


MPI_DECL1 long MPI_DECL2
	meiSqNodeConfigSet(MEISqNode	   node,
					   MEISqNodeConfig *config);

MPI_DECL1 long MPI_DECL2
	meiSqNodeFlashConfigGet(MEISqNode	    node,
							void 			*flash,
							MEISqNodeConfig *config);

MPI_DECL1 long MPI_DECL2
	meiSqNodeFlashConfigSet(MEISqNode	    node,
							void			*flash,
							MEISqNodeConfig *config);

MPI_DECL1 MPIControl MPI_DECL2
	meiSqNodeControl(MEISqNode node);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDownload(MEISqNode				  node,
					  MEISqNodeDownloadParams *params);

MPI_DECL1 long MPI_DECL2
	meiSqNodeVerify(MEISqNode				  node,
					  MEISqNodeDownloadParams *params);


MPI_DECL1 long MPI_DECL2
	meiSqNodeEventNotifyGet(MEISqNode	 node,
							MPIEventMask *eventMask,
							void		 *external);

MPI_DECL1 long MPI_DECL2
	meiSqNodeEventNotifySet(MEISqNode	 node,
							MPIEventMask eventMask,
							void		 *external);

MPI_DECL1 long MPI_DECL2
	meiSqNodeEventReset(MEISqNode	sqNode,
						MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	meiSqNodeFlashErase(MEISqNode	sqNode);

MPI_DECL1 long MPI_DECL2
	meiSqNodeFpgaFileNameVerify(MEISqNode  sqNode,
								const char*	fileName);
 
MPI_DECL1 long MPI_DECL2
	meiSqNodeFpgaDefaultFileName(MEISqNode				sqNode,
								 MEISqNodeFileName		*fileName);


MPI_DECL1 long MPI_DECL2
	meiSqNodeInfo(MEISqNode		node,
				  MEISqNodeInfo	*info); 

MPI_DECL1 long MPI_DECL2
	meiSqNodeMemory(MEISqNode node,
					void	  **memory);

MPI_DECL1 long MPI_DECL2
	meiSqNodeMemoryGet(MEISqNode  node,
					   void		  *dst,
					   const void *src,
					   long		  count);

MPI_DECL1 long MPI_DECL2
	meiSqNodeMemorySet(MEISqNode  node,
					   void		  *dst,
					   const void *src,
					   long		  count);

MPI_DECL1 long MPI_DECL2
	meiSqNodeNetworkObjectNext(MEISqNode            node, 
                               MEINetworkPort       port,
                               MEINetworkObjectInfo *info);

MPI_DECL1 long MPI_DECL2
	meiSqNodeNumber(MEISqNode	node,
					long		*number);

MPI_DECL1 long MPI_DECL2
	meiSqNodeStatus(MEISqNode		node,
					MEISqNodeStatus	*status);

MPI_DECL1 long MPI_DECL2
	meiSqNodeStatusClear(MEISqNode	node);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDigitalIn(MEISqNode		node,
					   long				bitStart,
					   long				bitCount,
					   unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDigitalOutSet(MEISqNode		node,
						   long				bitStart,
						   long				bitCount,
						   unsigned long	state,
						   MPI_BOOL			wait);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDigitalOutGet(MEISqNode		node,
						   long				bitStart,
						   long				bitCount,
						   unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeAnalogIn(MEISqNode	node,
					  long		channel,
					  long		*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeAnalogOutSet(MEISqNode	node,
						  long		channel,
						  long		state,
						  MPI_BOOL	wait);

MPI_DECL1 long MPI_DECL2
	meiSqNodeAnalogOutGet(MEISqNode	node,
						  long		channel,
						  long		*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentInfo(MEISqNode				node,
						  long					segment,
						  MEISqNodeSegmentInfo	*segmentInfo);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentDigitalIn(MEISqNode		node,
							  long			segment,
							  long			bitStart,
							  long			bitCount,
							  unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentDigitalOutSet(MEISqNode		node,
								  long			segment,
								  long			bitStart,
								  long			bitCount,
								  unsigned long	state,
								  MPI_BOOL		wait);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentDigitalOutGet(MEISqNode		node,
								  long			segment,
								  long			bitStart,
								  long			bitCount,
								  unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentAnalogIn(MEISqNode	node,
							 long		segment,
							 long		channel,
							 long		*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentAnalogOutSet(MEISqNode	node,
								 long		segment,
								 long		channel,
								 long		state,
								 MPI_BOOL	wait);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentAnalogOutGet(MEISqNode	node,
								 long		segment,
								 long		channel,
								 long		*state);

MPI_DECL1 long MPI_DECL2
	meiSqNodeUserDataGet(MEISqNode			node,
						 MEISqNodeUserData	*data);

MPI_DECL1 long MPI_DECL2
	meiSqNodeUserDataSet(MEISqNode					node,
						 const MEISqNodeUserData	*data);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentUserDataGet(MEISqNode					node,
								long						segment,
								MEISqNodeSegmentUserData	*data);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentUserDataSet(MEISqNode					node,
								long						segment,
								MEISqNodeSegmentUserData	*data);

#define MEISqNodeSEGMENT_MAX (32)
#define MEISqNodeSEGMENT_PARAMS_MAX (10)
#define MEISqNodeSEGMENT_MEMORY_MAX (30)

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentParamGet(MEISqNode sqNode,
							long segment,
							long paramStart,
							long paramCount,
							char *params );

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentParamSet(MEISqNode sqNode,
							long segment,
							long paramStart,
							long paramCount,
							char *params );

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentMemoryGet(MEISqNode sqNode,
							  long segment,
							  long memoryStart,
							  long memoryCount,
							  char *data);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentMemorySet(MEISqNode sqNode,
							  long segment,
							  long memoryStart,
							  long memoryCount,
							  char *data);

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentParamDefault( MEISqNode sqNode );

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentParamStore( MEISqNode sqNode );

MPI_DECL1 long MPI_DECL2
	meiSqNodeSegmentParamClear( MEISqNode sqNode );

/* 
	Drive specific node methods 
	
	These methods take drive specific processor structures cast as 
	void * data types. The drive specific structures can be found in 
	the	sqNode library include directory (C:\MEI)\XMP\sqNodeLib\include 
	directory.

	Node/Drive specific include files are names according to drive
	Vendor_Device type.
	
 */
MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveConfigGet(MEISqNode	node,
							long		driveIndex,		/* relative to the node */
							void		*config);		/* node specific        */

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveConfigSet(MEISqNode	node,
							long		driveIndex,		/* relative to the node */
							void		*config);		/* node specific        */

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveInfo(MEISqNode			node,
					   long					driveIndex,
					   MEISqNodeDriveInfo	*info,
					   void					*external);	/* node specific */

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveParamGet(MEISqNode				node,
						long						driveIndex,
						long						param,
						MEIDriveMapParamType		paramType,
						MEIDriveMapParamValue		*value);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveParamSet(MEISqNode				node,
						   long						driveIndex,
						   long						param,
						   MEIDriveMapParamType	paramType,
						   MEIDriveMapParamValue	*value);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveParamListGet(MEISqNode				node,
							   long						driveIndex,
							   long						size,
							   long						*paramList,
							   MEIDriveMapParamType		*paramTypes,
							   MEIDriveMapParamValue	*paramValues);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamListSet(MEISqNode				node,
							   long						driveIndex,
							   long						size,
							   long						*paramList,
							   MEIDriveMapParamType		*paramTypes,
							   MEIDriveMapParamValue	*paramValues);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamStore(MEISqNode	sqNode,
							 long		driveIndex);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamRestore(MEISqNode sqNode,
							   long		 driveIndex);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamClear(MEISqNode	sqNode,
							 long		driveIndex);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamReload(MEISqNode	sqNode,
							  long		driveIndex);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveParamCalculate(MEISqNode	sqNode,
							     long		driveIndex);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveMapParamFileGet(MEISqNode	sqNode,
								MEIDriveMap	driveMap,
								long		driveIndex,
								char		*driveConfigFilename,
								long		append);

MPI_DECL1 long MPI_DECL2  
	meiSqNodeDriveMapParamFileSet(MEISqNode	sqNode,
								MEIDriveMap	driveMap,
								long		driveIndex,
								char		*driveConfigFilename,
								MEISqNodeDriveParamCallback callback,
								long			warning);


/* Drive Map File Functions */
MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMapParamCount(MEISqNode	sqNode,
								MEIDriveMap	driveMap,
								long		driveIndex,
								long		*paramsCount);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMapParamList(MEISqNode		sqNode,
								MEIDriveMap		driveMap,
								long			driveIndex,
								long			paramsCount,
								MEIDriveMapParamInfo *driveParamInfo);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMapConfigCount(MEISqNode	sqNode,
								MEIDriveMap	driveMap,
								long		driveIndex,
								long		*configCount);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMapConfigList(MEISqNode		sqNode,
								MEIDriveMap		driveMap,
								long			driveIndex,
								long			configCount,
								long			*configList);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMonitorConfigGet(MEISqNode					node,
								   long							driveIndex,	/* relative to the node */
								   MEISqNodeDriveMonitorConfig	*config);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMonitorConfigSet(MEISqNode					node,
								   long							driveIndex,	/* relative to the node */
								   MEISqNodeDriveMonitorConfig	*config);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMonitor(MEISqNode				node,
						  long					driveIndex,	/* relative to the node */
						  MEISqNodeMonitorValue	*value);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDriveMonitorInfo(MEISqNode					node,
						 long						driveIndex,
						 MEISqNodeDriveMonitorInfo	*info); 

MPI_DECL1 long MPI_DECL2
	meiSqNodeDigitalInPtr(MEISqNode	sqNode,
						long		bit,
						long		*mask,
						long		**ptr);

MPI_DECL1 long MPI_DECL2
	meiSqNodeDigitalOutPtr(MEISqNode	sqNode,
						long		bit,
						long		*mask,
						long		**ptr);

MPI_DECL1 long MPI_DECL2
	meiSqNodeAnalogInPtr(MEISqNode	sqNode,
						long		channel,
						long		*mask,
						long		**ptr);

MPI_DECL1 long MPI_DECL2
	meiSqNodeAnalogOutPtr(MEISqNode	sqNode,
						long		channel,
						long		*mask,
						long		**ptr);


typedef struct MEISqNodeLibAsynqTxRxHostAddress {
	long txAddr;
	long rxAddr;
	long txTcbAddr;
	long rxTcbAddr;
	long asynqBuffAddress; /*This is the Asynq Buffer start address */
}MEISqNodeLibAsynqTxRxHostAddress ;

typedef enum MEISqNodeControllerTxPort{
	MEISqNodeControllerTxPortINVALID = -1,
	MEISqNodeControllerTxPortOUT_PORT,
	MEISqNodeControllerTxPortIN_PORT,
	MEISqNodeControllerTxPortLAST
}MEISqNodeControllerTxPort;

MPI_DECL1 long MPI_DECL2
	mpiSqnodeLibAsynqTxRxHostPtrs(MPIControl control, 
				MEISqNodeLibAsynqTxRxHostAddress *txRxAsynqPtrs);

MPI_DECL1 long MPI_DECL2
	mpiSqNodeDownloadAsynqInit(MPIControl control, long nodeNumber );

MPI_DECL1 long MPI_DECL2
	mpiSqNodeControllerTxPort(MPIControl control, 
							long nodeNumber,
							MEISqNodeControllerTxPort *contollerTxport);
MPI_DECL1 long MPI_DECL2
	mpiSqNodeDownloadServiceCommand(MEISqNode			sqNode,
									MEISqNodeCommand	*command,
									MEISqNodeResponse	*response,
									long timeOutMillSec);
MPI_DECL1 long MPI_DECL2
	mpiSqNodeDownloadPipeLineServiceCommand(MEISqNode			sqNode,
											MEISqNodeCommand	*command,
											MEISqNodeResponse	*response,
											void * txdataBuffer,
											void *rxDataBuffer,
											long count);
MPI_DECL1 long MPI_DECL2
	mpiSqNodeDownloadUART_RxData( MPIControl control,
								long nodeNumber,
								char *rxData,
								long rxDataSize,
								long *rxReceivedCount);
MPI_DECL1 long MPI_DECL2
	mpiSqNodeDownloadAsynqDriveInit(MPIControl control,
									long nodeNumber);

#if defined(__cplusplus)
}
#endif

#endif
