/* Synq.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined (__SYNQNET__)
#define __SYNQNET__

#if defined(__cplusplus)
extern "C" {
#endif

/* #defines and enums */
#define MEIXmpSynqNetAddrMaster					(0xFD)
#define MEIXmpSynqNetAddrBroadcast				(0xFF)
#define MEIXmpSynqNetAddrUndiscovered			(0xFE)

#define MEIXmpSynqNetTypeBitShift				(0)
#define MEIXmpSynqNetDestAddrBitShift			(8)
#define MEIXmpSynqNetSrcAddrBitShift			(8)

#define MEIXmpSynqNetAsynqDelayShift			(8)

#define MEIXmpSynqNetMessageTypeSynq			(0x01)	/* Phase: Cyclic */
#define MEIXmpSynqNetMessageTypeCommand			(0x04)	/* Phase: Cyclic */
#define MEIXmpSynqNetMessageTypeControl			(0x05)	/* Phase: Cyclic/Discovery */
#define MEIXmpSynqNetMessageTypeFeedback		(0x80)	/* Phase: Cyclic */
#define MEIXmpSynqNetMessageTypeStatus			(0x82)	/* Phase: Cyclic/Discovery */
#define MEIXmpSynqNetMessageTypeProbe			(0x20)	/* Phase: Validate */
#define MEIXmpSynqNetMessageTypeResetRequest	(0x21)	/* Phase: Ready */
#define MEIXmpSynqNetMessageTypeResetComplete	(0x22)	/* Phase: Reset */
#define MEIXmpSynqNetMessageTypeChange			(0x23)	/* Phase: Cyclic */
#define MEIXmpSynqNetMessageTypeTest			(0x24)	/* Phase: Cyclic */
#define MEIXmpSynqNetMessageTypeRestartRequest	(0x25)	/* Phase: Restart */

/* SynqNet TCB for DMAing data to external memory.  Transfers use DMA6.  Receives use DMA7.  Both of these
	DMA channels are already in use by the ADCs, so we will use these channels when the ADC is not using them.
*/
typedef struct MEIXmpSynqNetTCB
{	long tcbEC;				/* External Count */
	long tcbEM;				/* External Modifier */
	volatile long *tcbEI;	/* External Index */
	long tcbGP;				/* General Purpose - Not Used */
	long *tcbCP;			/* Chain Pointer */
	long tcbC;				/* Internal Count */
	long tcbIM;				/* Inernal Modifier */
	volatile long *tcbII;	/* Internal Index */
} MEIXmpSynqNetTCB;

typedef struct MEIXmpSynqNetCopy
{	MEIXmpSynqNetTCB Tx;
	MEIXmpSynqNetTCB Rx;
} MEIXmpSynqNetCopy;

/* lower 16 bits of Header_Cmd/StatusFlags contain header info: lower 8 bits - telegram type, upper 8 bits - address */
#define MEIXmpSynqNetHeaderMask			(0x0000FFFF)

/* SynqNet Service Channel Flags - ALL FLAGS OCCUPY UPPER 16 BITS of Header_Cmd/StatusFlags */
typedef enum MEIXmpSynqNetServiceCommandFlags
{	MEIXmpSynqNetServiceFlagsIgnore = (int)(0x0 << 16),						/* Bit 0 */
	MEIXmpSynqNetServiceFlagsValid = (int)(0x1 << 16),						/* Bit 0 */
	MEIXmpSynqNetServiceFlagsRead = (int)(0x0 << 16),						/* Bit 1 */
	MEIXmpSynqNetServiceFlagsWrite = (int)(0x2 << 16),						/* Bit 1 */
	MEIXmpSynqNetServiceFlagsSize8Bit = (int)(0x0 << 16),					/* Bits 2-3 */
	MEIXmpSynqNetServiceFlagsSize16Bit = (int)(0x4 << 16),					/* Bits 2-3 */
	MEIXmpSynqNetServiceFlagsSize24Bit = (int)(0x8 << 16),					/* Bits 2-3 */
	MEIXmpSynqNetServiceFlagsSize32Bit = (int)(0xC << 16),					/* Bits 2-3 */
	MEIXmpSynqNetServiceFlagsMemorySpaceData = (int)(0x00 << 16),			/* Bits 4-5 */
	MEIXmpSynqNetServiceFlagsMemorySpaceProgram = (int)(0x10 << 16),		/* Bits 4-5 */
	MEIXmpSynqNetServiceFlagsMemorySpaceIO = (int)(0x20 << 16),				/* Bits 4-5 */
	MEIXmpSynqNetServiceFlagsMemorySpaceDirect = (int)(0x30 << 16),			/* Bits 4-5 */
	MEIXmpSynqNetServiceFlagsPipelineOn = (int)(0x40 << 16),				/* Bit 6 */
	MEIXmpSynqNetServiceFlagsPipelineOff = (int)(0x00 << 16),				/* Bit 6 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor0 = (int)(0x0000 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor1 = (int)(0x0100 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor2 = (int)(0x0200 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor3 = (int)(0x0300 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor4 = (int)(0x0400 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor5 = (int)(0x0500 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor6 = (int)(0x0600 << 16),	/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectProcessor7 = (int)(0x0700 << 16),	/* Bits 8-15 */

	
	MEIXmpSynqNetServiceFlagsDeviceSelectFPGA = (int)(0xFE00 << 16),		/* Bits 8-15 */
	MEIXmpSynqNetServiceFlagsDeviceSelectNULL = (int)(0xFF00 << 16)			/* Bits 8-15 */
} MEIXmpSynqNetServiceCommandFlags;

typedef enum MEIXmpSynqNetServiceStatusFlags
{	MEIXmpSynqNetServiceStatusFlagsNotDone = (int)(0x0 << 16),			/* Bit 0 */
	MEIXmpSynqNetServiceStatusFlagsDone = (int)(0x1 << 16),				/* Bit 0 */
	MEIXmpSynqNetServiceStatusFlagsSuccess = (int)(0x0 << 16),			/* Bit 1 */
	MEIXmpSynqNetServiceStatusFlagsError = (int)(0x2 << 16),			/* Bit 1 */
	MEIXmpSynqNetServiceStatusFlagsCommandOK = (int)(0x0 << 16),		/* Bit 2 */
	MEIXmpSynqNetServiceStatusFlagsCommandUnsupported = (int)(0x4 << 16),/* Bit 2 */

	MEIXmpSynqNetServiceStatusFlagsSequenceERROR = (int)(0x2000 << 16),	 /* Bit 13 - not really defined by synqnet spec, but used for sequence error detection */
	MEIXmpSynqNetServiceStatusFlagsTimeoutRESPONSE = (int)(0x4000 << 16),/* Bit 14 - not really defined by synqnet spec, but used for firmware timeout */
	MEIXmpSynqNetServiceStatusFlagsTimeoutREADY = (int)(0x8000 << 16)	 /* Bit 15 - not really defined by synqnet spec, but used for firmware timeout */
} MEIXmpSynqNetServiceStatusFlags;

typedef enum MEIXmpSynqNetPacketSynqFlags {
	MEIXmpSynqNetPacketSynqFlagsRESTART = (int)0x40000000,
	MEIXmpSynqNetPacketSynqFlagsSYNQMODE = (int)0x80000000
} MEIXmpSynqNetPacketSynqFlags;

/* MEIXmpSynqNetServiceCommand MUST ALWAYS be the fist structure in the telegram */
typedef struct MEIXmpSynqNetServiceCommand
{	long Header;			/* lower 16 bits Header / upper 16 bits Command Flags */
	long Address;			/* 32 bit address */
	long Data;				/* 32 bit data */
} MEIXmpSynqNetServiceCommand;

/* MEIXmpSynqNetServiceStatus MUST ALWAYS be the fist structure in the telegram */
typedef struct MEIXmpSynqNetServiceStatus
{	long Header;				/* lower 16 bits Header / upper 16 bits Status Flags */
	long Response;				/* 32 bit response data */
} MEIXmpSynqNetServiceStatus;

/* Packets - CRC word not included here since it is added and deleted by Rincon */
/* Synq */
typedef struct MEIXmpSynqNetPacketSynq
{	long Header;		/* bits 0-7: Type 0x01  bits 8-15: node Address 0xFF (broadcast)  Bits 16-31: Flags */
	long CycleCount;
} MEIXmpSynqNetPacketSynq;

/* Probe */
typedef struct MEIXmpSynqNetPacketProbe
{	long Header;		/* bits 0-7: Type 0x20  bits 8-15: node Address 0xFF (broadcast)  Bits 16-31: Null */
} MEIXmpSynqNetPacketProbe;

/* Change */
typedef struct MEIXmpSynqNetPacketChange
{	long Header;		/* bits 0-7: Type 0x23  bits 8-15: node Address 0xFF (broadcast)  Bits 16-31: Null */
} MEIXmpSynqNetPacketChange;

/* ResetRequest */
typedef struct MEIXmpSynqNetPacketResetRequest
{	long Header;		/* bits 0-7: Type 0x21  bits 8-15: node Address 0xFF (broadcast)  Bits 16-31: Null */
} MEIXmpSynqNetPacketResetRequest;

/* ResetComplete */
typedef struct MEIXmpSynqNetPacketResetComplete
{	long Header;		/* bits 0-7: Type 0x22  bits 8-15: node Address 0xFF (broadcast)  Bits 16-31: Null */
} MEIXmpSynqNetPacketResetComplete;

/* AsynqControl (ServiceChannel) */
typedef struct MEIXmpSynqNetPacketAsynqControl
{	long Header;		/* bits 0-7: Type 0x05  bits 8-15: node Address  bits 16-31: ServiceChannel Command Flags */
	long ServiceChannelAddress;
	long ServiceChannelData;
} MEIXmpSynqNetPacketAsynqControl;

/* AsynqStatus (ServiceChannel) */
typedef struct MEIXmpSynqNetPacketAsynqStatus
{	long Header;		/* bits 0-7: Type 0x82  bits 8-15: node Address  bits 16-31: ServiceChannel Status Flags */
	long ServiceChannelResponse;
} MEIXmpSynqNetPacketAsynqStatus;



typedef struct MEIXmpSynqNetAsynqTx
{	MEIXmpSynqNetPacketAsynqControl		Control;
	MEIXmpSynqNetPacketProbe			Probe;
	MEIXmpSynqNetPacketResetRequest		ResetRequest;
	MEIXmpSynqNetPacketResetComplete	ResetComplete;
} MEIXmpSynqNetAsynqTx;

typedef struct MEIXmpSynqNetAsynqRx
{	MEIXmpSynqNetPacketAsynqStatus		Status;
	MEIXmpSynqNetPacketProbe			Probe;
	MEIXmpSynqNetPacketResetRequest		ResetRequest[2];
	MEIXmpSynqNetPacketResetComplete	ResetComplete[2];
} MEIXmpSynqNetAsynqRx;

typedef struct MEIXmpSynqNetMessageAsynq
{	MEIXmpSynqNetAsynqTx	Tx;
	MEIXmpSynqNetAsynqRx	Rx;
} MEIXmpSynqNetMessageAsynq;


#if defined(__cplusplus)
}
#endif

#endif /* __SYNQNET__ */
