/* synqnet.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_SYNQNET_H)
#define	_SYNQNET_H

/* SynqNet handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEISynqNet *MEISynqNet;
#if defined(__cplusplus)
}
#endif

#include "stdmei.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define MEISynqNetMaxNODE_COUNT          (MEIXmpMaxSynqNetBlocks)      /* 32 */
#define MEISynqNetMaxMOTORS				 (MEIXmpMAX_Motors)			   /* 32 */
#define MEISynqNetCableHOP_COUNT         (MEISynqNetMaxNODE_COUNT + 1) /* 33 */

/* SynqNet Cables */
typedef enum MEISynqNetCableStatus {
	MEISynqNetCableStatusINVALID = -1,

	MEISynqNetCableStatusGOOD,
	MEISynqNetCableStatusBAD_UPSTREAM,
	MEISynqNetCableStatusBAD_DOWNSTREAM,
	MEISynqNetCableStatusBAD,

	MEISynqNetCableStatusLAST,
	MEISynqNetCableStatusFIRST = MEISynqNetCableStatusINVALID + 1
} MEISynqNetCableStatus;

typedef struct MEISynqNetCableList {
	long count;
	long cableNumber[MEISynqNetCableHOP_COUNT];
} MEISynqNetCableList;

/* SynqNet Config */
typedef struct MEISynqNetCableLength {	/* lengths in meters */
	long minimum;		
	long nominal;		
	long maximum;
} MEISynqNetCableLength;

typedef enum MEISynqNetRecoveryMode {
	MEISynqNetRecoveryModeINVALID = -1,

	MEISynqNetRecoveryModeDISABLED,
	MEISynqNetRecoveryModeSINGLE_SHOT,
	MEISynqNetRecoveryModeAUTO_ARM,

	MEISynqNetRecoveryModeLAST,
	MEISynqNetRecoveryModeFIRST = MEISynqNetRecoveryModeINVALID + 1
} MEISynqNetRecoveryMode;

typedef struct MEISynqNetConfig {
	MEISynqNetRecoveryMode recoveryMode;
	MEISynqNetCableLength  cableLength[MEISynqNetCableHOP_COUNT];
} MEISynqNetConfig;

/* SynqNet Info */
typedef struct MEISynqNetInfo {
	MEINetworkType        networkType;
	long                  nodeCount;
	long                  nodeOffset;
	MEISynqNetCableLength cableLength[MEISynqNetCableHOP_COUNT]; /* measured length */
} MEISynqNetInfo;

/* SynqNet Packet Configuration */
#define MEISynqNetMaxMotorFEEDBACK_PRIMARY_COUNT	(1)	 
#define MEISynqNetMaxMotorFEEDBACK_SECONDARY_COUNT	(MEISqNodeMaxFEEDBACK_SECONDARY)
#define MEISynqNetMaxMotorCAPTURE_COUNT				(MEIXmpMaxCapturesPerMotor) /*  2 */
#define MEISynqNetMaxMotorCOMPARE_COUNT				(MEIXmpMaxComparesPerMotor) /*  2 */
#define MEISynqNetMaxMotorPULSE_ENGINE_COUNT		(1)	
#define MEISynqNetMaxMotorPROBE_COUNT				(4)

#define MEISynqNetMaxNodeMOTORS						(MEISqNodeMaxMOTORS)   

typedef enum MEISynqNetResourceCommand {
	MEISynqNetResourceCommandINVALID   = -1,

	MEISynqNetResourceCommandNONE,
	MEISynqNetResourceCommandDAC_ONE,
	MEISynqNetResourceCommandDAC_TWO,
	MEISynqNetResourceCommandDEMAND_A,
	MEISynqNetResourceCommandDEMAND_AB,
	MEISynqNetResourceCommandDEMAND_ABC,

	MEISynqNetResourceCommandLAST,
	MEISynqNetResourceCommandFIRST     = MEISynqNetResourceCommandINVALID + 1
} MEISynqNetResourceCommand;

typedef enum MEISynqNetResourceIoBits {
	MEISynqNetResourceIoBitsINVALID   = -1,

	MEISynqNetResourceIoBitsNONE,
	MEISynqNetResourceIoBits16,
	MEISynqNetResourceIoBits48,

	MEISynqNetResourceIoBitsLAST,
	MEISynqNetResourceIoBitsFIRST     = MEISynqNetResourceIoBitsINVALID + 1
} MEISynqNetResourceIoBits;

typedef enum MEISynqNetResourceMonitor {
	MEISynqNetResourceMonitorINVALID  = -1,

	MEISynqNetResourceMonitorNONE,
	MEISynqNetResourceMonitorAB,
	MEISynqNetResourceMonitorABCD,

	MEISynqNetResourceMonitorLAST,
	MEISynqNetResourceMonitorFIRST    = MEISynqNetResourceMonitorINVALID + 1
} MEISynqNetResourceMonitor;

typedef enum MEISynqNetResourceProbeDepth {
	MEISynqNetResourceProbeDepthINVALID  = -1,

	MEISynqNetResourceProbeDepthNONE,
	MEISynqNetResourceProbeDepthTWO,
	MEISynqNetResourceProbeDepthFOUR,
	MEISynqNetResourceProbeDepthEIGHT,
	MEISynqNetResourceProbeDepthSIXTEEN,

	MEISynqNetResourceProbeDepthLAST,
	MEISynqNetResourceProbeDepthFIRST    = MEISynqNetResourceProbeDepthINVALID + 1
} MEISynqNetResourceProbeDepth;

typedef struct MEISynqNetPacketCfgProbe {	
	long                         count;						   
	MEISynqNetResourceProbeDepth depth[MEISynqNetMaxMotorPROBE_COUNT];						   
} MEISynqNetPacketCfgProbe;

typedef struct MEISynqNetPacketCfgMotor {	
	MEISynqNetResourceCommand  command;	
	long					   pulseEngineCount;
	long					   feedbackPrimaryCount;
	long					   captureCount;						   
	long                       compareCount;						   
	MEISynqNetResourceIoBits   ioInput;
	MEISynqNetResourceIoBits   ioOutput;			
	MEISynqNetResourceMonitor  monitor;
	MEISynqNetPacketCfgProbe   probe;
} MEISynqNetPacketCfgMotor;

typedef struct MEISynqNetPacketCfgIo {	
	long digitalInCount;
	long digitalOutCount;
	long analogInCount;
	long analogOutCount;
} MEISynqNetPacketCfgIo;

typedef struct MEISynqNetPacketCfgNode {
	MEISynqNetPacketCfgMotor motor[MEISynqNetMaxNodeMOTORS];
	MEISynqNetPacketCfgIo	 io;
	long					 feedbackSecondaryCount;
} MEISynqNetPacketCfgNode;

typedef struct MEISynqNetPacketCfg {
	MEISynqNetPacketCfgNode node[MEISynqNetMaxNODE_COUNT];
} MEISynqNetPacketCfg;

/* SynqNet Status */
#define MEISynqNetNodeMaskELEMENTS	(1)
typedef long MEISynqNetFailedNodeMask[MEISynqNetNodeMaskELEMENTS];
typedef MEISynqNetFailedNodeMask  MEISynqNetShutdownNodeMask;

typedef struct MEISynqNetStatusCrcError {
	long port[MEINetworkPortLAST];
} MEISynqNetStatusCrcError;

typedef enum MEISynqNetState {
	MEISynqNetStateINVALID = -1,

	MEISynqNetStateDISCOVERY,
	MEISynqNetStateASYNQ,
	MEISynqNetStateSYNQ,
	MEISynqNetStateSYNQ_RECOVERING,

	MEISynqNetStateLAST,
	MEISynqNetStateFIRST = MEISynqNetStateINVALID + 1
} MEISynqNetState;

typedef struct MEISynqNetStatus {
	MEISynqNetState				state;
	MPI_BOOL					topologySaved;		/* TRUE/FALSE */
	MEISynqNetStatusCrcError	crcError;
	MPIEventMask				eventMask;
	MEISynqNetFailedNodeMask	failedNodeMask;
	MEISynqNetShutdownNodeMask	shutdownNodeMask;
} MEISynqNetStatus;

/* SynqNet Timing */
typedef struct MEISynqNetTiming {
	double controllerFreq;      /* kHz */
	double controllerPeriod;    /* uS  */

	long   txTime;				/*  %  */
	double calculationLimit;	/* uS  */
	double calculationTime;		/* uS  */
	double calculationSlack;	/* uS  */

	long   bandwidthUsage;      /*  %  */

	struct {
		double synq;            /* uS  - synq packet + spacing         */
		double demand;          /* uS  - sum demand packets + spacing  */
		double control;         /* uS  - sum control packets + spacing */
		
		double total;           /* uS  */
	} downstream;

	struct {
		double feedback;        /* uS  - sum feedback packets + spacing */
		double status;          /* uS  - sum status packets + spacing   */

		double total;           /* uS  */
	} upstream;

	struct {
		double updateFreq;		/* kHz */
		double updatePeriod;	/* uS  */
	
		double demandLatency;	/* uS  */
		double feedbackLatency;	/* uS  */
		double latencyOverhead;	/* uS  */		
		double controlLatency;  /* uS  */

		double discoveryLatencyTolerance;    /* +/- uS per network discovery */
	} node[MEISynqNetMaxNODE_COUNT];

} MEISynqNetTiming;


/* SynqNet Messages */
typedef enum {
	MEISynqNetMessageFIRST = mpiMessageID(MEIModuleIdSYNQNET, 0),

	MEISynqNetMessageSYNQNET_INVALID,
	MEISynqNetMessageMAX_NODE_ERROR,
	MEISynqNetMessageSTATE_ERROR,

	MEISynqNetMessageCOMM_ERROR,
	MEISynqNetMessageCOMM_ERROR_CRC,
	MEISynqNetMessageCOMM_ERROR_RX,
	MEISynqNetMessageCOMM_ERROR_RX_LEN,
	MEISynqNetMessageCOMM_ERROR_RX_FIFO,
	MEISynqNetMessageCOMM_ERROR_RX_DRIBBLE,
	MEISynqNetMessageCOMM_ERROR_RX_CRC,

	MEISynqNetMessageINTERFACE_NOT_FOUND,
	MEISynqNetMessageTOPOLOGY_MISMATCH,
	MEISynqNetMessageTOPOLOGY_MISMATCH_FLASH,

	MEISynqNetMessageRESET_REQ_TIMEOUT,
	MEISynqNetMessageRESET_ACK_TIMEOUT,
	MEISynqNetMessageDISCOVERY_TIMEOUT,
	MEISynqNetMessageNO_NODES_FOUND,
	MEISynqNetMessageNO_TIMING_DATA_AVAIL,

	MEISynqNetMessageINTERNAL_BUFFER_OVERFLOW,
	MEISynqNetMessageINVALID_MOTOR_COUNT,
	MEISynqNetMessageINVALID_AUX_ENC_COUNT,
	MEISynqNetMessageINCOMPLETE_MOTOR,
	MEISynqNetMessageINVALID_COMMAND_CFG,
	MEISynqNetMessageINVALID_PULSE_ENGINE_COUNT,
	MEISynqNetMessageINVALID_ENCODER_COUNT,
	MEISynqNetMessageINVALID_CAPTURE_COUNT,
	MEISynqNetMessageINVALID_COMPARE_COUNT,
	MEISynqNetMessageINVALID_INPUT_COUNT,
	MEISynqNetMessageINVALID_OUTPUT_COUNT,
	MEISynqNetMessageINVALID_MONITOR_CFG,
	MEISynqNetMessageINVALID_ANALOG_IN_COUNT,
	MEISynqNetMessageINVALID_DIGITAL_IN_COUNT,
	MEISynqNetMessageINVALID_DIGITAL_OUT_COUNT,
	MEISynqNetMessageINVALID_ANALOG_OUT_COUNT,

	MEISynqNetMessageLINK_NOT_IDLE,
	MEISynqNetMessageIDLE_LINK_UNKNOWN,
	MEISynqNetMessageRING_ONLY,
	MEISynqNetMessageRECOVERING,

	MEISynqNetMessageCABLE_LENGTH_UNSUPPORTED,
	MEISynqNetMessageCABLE_LENGTH_TIMEOUT,
	MEISynqNetMessageCABLE_LENGTH_MISMATCH,
	MEISynqNetMessageCABLE_LENGTH_INVALID_NOMINAL,
	MEISynqNetMessageCABLE_LENGTH_INVALID_MIN,
	MEISynqNetMessageCABLE_LENGTH_INVALID_MAX,

	MEISynqNetMessageNODE_FPGA_VERSION,
	MEISynqNetMessageMAX_MOTOR_ERROR,
	MEISynqNetMessagePLL_ERROR,
	MEISynqNetMessageNODE_INIT_FAIL,

	MEISynqNetMessageTOPOLOGY_CLEAR,
	MEISynqNetMessageTOPOLOGY_SAVED,
	MEISynqNetMessageTOPOLOGY_AMPS_ENABLED,

	MEISynqNetMessageNODE_MAC_VERSION,
	MEISynqNetMessageADC_SAMPLE_FAILURE,

	MEISynqNetMessageSCHEDULING_ERROR,

	MEISynqNetMessageINVALID_PROBE_CFG,
	MEISynqNetMessageINVALID_PROBE_DEPTH,
	MEISynqNetMessageSAMPLE_PERIOD_NOT_MULTIPLE,
	MEISynqNetMessageNODE_LATENCY_EXCEEDED,

	MEISynqNetMessageHOT_RESTART_FAIL_NOT_SYNQ_STATE,
	MEISynqNetMessageHOT_RESTART_FAIL_RECOVERING,
	MEISynqNetMessageHOT_RESTART_FAIL_TEST_PACKET,
	MEISynqNetMessageHOT_RESTART_FAIL_ADDRESS_ASSIGNMENT,
	MEISynqNetMessageHOT_RESTART_NOT_ALL_NODES_RESTARTED,

	MEISynqNetMessageSHUTDOWN_NODES_NONCONSECUTIVE,
	MEISynqNetMessageSHUTDOWN_NODES_STRANDED,
	MEISynqNetMessageSHUTDOWN_RECOVERY_DISABLED,

	MEISynqNetMessageSHUTDOWN_REQUESTED_NODE_INVALID,
	MEISynqNetMessageNODE_BOOT_FPGA_VERSION,
	MEISynqNetMessageNODE_FPGA_LAB_VERSION,
	MEISynqNetMessageNODE_MAC_LAB_VERSION,
	MEISynqNetMessageLAST
} MEISynqNetMessage;

typedef enum {
	MEISynqNetTraceFIRST		= MEITraceLAST << 1,

	MEISynqNetTraceDYNA_ALLOC	= MEISynqNetTraceFIRST << 0,
	MEISynqNetTraceDYNA_FREE	= MEISynqNetTraceFIRST << 1,
	MEISynqNetTraceINIT			= MEISynqNetTraceFIRST << 2,
	MEISynqNetTraceRESET		= MEISynqNetTraceFIRST << 3,
	MEISynqNetTraceTIMING		= MEISynqNetTraceFIRST << 4,
	MEISynqNetTraceSERVICE_CMD	= MEISynqNetTraceFIRST << 5,
	MEISynqNetTraceFLOWCTRL		= MEISynqNetTraceFIRST << 6,
	MEISynqNetTraceMAP			= MEISynqNetTraceFIRST << 7,
	MEISynqNetTraceUNMAP		= MEISynqNetTraceFIRST << 8,

	MEISynqNetTraceLAST			= MEISynqNetTraceFIRST << 9
} MEISynqNetTrace;


/* Create/Delete/Validate */
MPI_DECL1 MEISynqNet MPI_DECL2
	meiSynqNetCreate(MPIControl	control,
					 long		number);

MPI_DECL1 long MPI_DECL2
	meiSynqNetDelete(MEISynqNet synqNet);

MPI_DECL1 long MPI_DECL2
	meiSynqNetValidate(MEISynqNet synqNet);

/* SynqNet */
MPI_DECL1 long MPI_DECL2
	meiSynqNetConfigGet(MEISynqNet       synqNet,
						MEISynqNetConfig *config);

MPI_DECL1 long MPI_DECL2
	meiSynqNetConfigSet(MEISynqNet       synqNet,
						MEISynqNetConfig *config);

MPI_DECL1 MPIControl MPI_DECL2
	meiSynqNetControl(MEISynqNet synqNet);

MPI_DECL1 long MPI_DECL2
	meiSynqNetEventNotifyGet(MEISynqNet	  synqNet,
							 MPIEventMask *eventMask,
							 void		  *external);

MPI_DECL1 long MPI_DECL2
	meiSynqNetEventNotifySet(MEISynqNet	  synqNet,
							 MPIEventMask eventMask,
							 void		  *external);

MPI_DECL1 long MPI_DECL2
	meiSynqNetEventReset(MEISynqNet		synqNet,
						 MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	meiSynqNetFlashConfigGet(MEISynqNet       synqNet,
							 void			  *flash,
							 MEISynqNetConfig *config);

MPI_DECL1 long MPI_DECL2
	meiSynqNetFlashConfigSet(MEISynqNet       synqNet,
							 void			  *flash,
							 MEISynqNetConfig *config);

MPI_DECL1 long MPI_DECL2
	meiSynqNetFlashTopologySave(MEISynqNet synqNet, MEIFlash flash);

MPI_DECL1 long MPI_DECL2
	meiSynqNetFlashTopologyClear(MEISynqNet synqNet, MEIFlash flash);

MPI_DECL1 long MPI_DECL2
	meiSynqNetInfo(MEISynqNet	  synqNet,
				   MEISynqNetInfo *info);

MPI_DECL1 long MPI_DECL2				
	meiSynqNetInit(MEISynqNet	synqNet); 

MPI_DECL1 long MPI_DECL2				
	meiSynqNetIdleCableListGet(MEISynqNet          synqNet,
							   MEISynqNetCableList *idleCable);

MPI_DECL1 long MPI_DECL2				
	meiSynqNetIdleCableStatus(MEISynqNet            synqNet,
						      long                  cableNumber,
							  MEISynqNetCableStatus *cableStatus);

MPI_DECL1 long MPI_DECL2
	meiSynqNetNumber(MEISynqNet	synqNet,
					 long		*number);

MPI_DECL1 long MPI_DECL2
	meiSynqNetNetworkObjectNext(MEISynqNet			  synqNet, 
								MEINetworkPort		  port,
								MEINetworkObjectInfo  *info);

MPI_DECL1 long MPI_DECL2
    meiSynqNetPacketConfigGet(MEISynqNet          synqNet,
	 						  MEISynqNetPacketCfg *config);

/* WARNING: meiSynqNetPacketConfigSet(...) and 
 * meiSynqNetPacketFlashConfigSet(...) are  low-level network routines that 
 * must clear other controller object configurations.  This method should be   
 * run before configuring most MPI objects.  Please refer to the online
 * documentation for more information.
 */
MPI_DECL1 long MPI_DECL2							
    meiSynqNetPacketConfigSet(MEISynqNet          synqNet,	
							  MEISynqNetPacketCfg *config);

MPI_DECL1 long MPI_DECL2
    meiSynqNetPacketFlashConfigGet(MEISynqNet          synqNet,
								   void				   *flash,
	 							   MEISynqNetPacketCfg *config);

MPI_DECL1 long MPI_DECL2							
    meiSynqNetPacketFlashConfigSet(MEISynqNet          synqNet,	
				   				   void			       *flash,
								   MEISynqNetPacketCfg *config);

MPI_DECL1 long MPI_DECL2				
	meiSynqNetShutdown(MEISynqNet synqNet);	
															
MPI_DECL1 long MPI_DECL2
	meiSynqNetStatus(MEISynqNet		  synqNet,
					 MEISynqNetStatus *status);

MPI_DECL1 long MPI_DECL2
	meiSynqNetTiming(MEISynqNet	      synqNet,
					 MEISynqNetTiming *timing);

MPI_DECL1 long MPI_DECL2
	meiSynqNetNodeRestart(MEISynqNet synqNet);

MPI_DECL1 long MPI_DECL2
	meiSynqNetNodeShutdown(MEISynqNet 					synqNet,
					   	MEISynqNetShutdownNodeMask 	nodeMask);

MPI_DECL1 long MPI_DECL2
	getSynqNetInPortBlockCount(MEISynqNet synqnet, 
								   long *inPortBlockCount);

MPI_DECL1 long MPI_DECL2
	blockCountsGet(MEISynqNet synqnet,
						long *InPortBlockCount,
						long *OutPortBlockCount);

MPI_DECL1 long MPI_DECL2
	nodeLinkStatus(MPIControl     control,
						   long           nodeNumber,
						   MEINetworkPort port,
						   MPI_BOOL       *linkStatusUp);  /* TRUE/FALSE */

MPI_DECL1 long MPI_DECL2
	nodeRepeaterEnableSet(MPIControl     control,
						  long           nodeNumber,
						  MEINetworkPort port,
						  MPI_BOOL       enable);  /* TRUE/FALSE */

MPI_DECL1 long MPI_DECL2
	testPacketXmpSend(MEISynqNet synqNet,
						long       port);

MPI_DECL1 long MPI_DECL2
	testPacketXmpCheck(MEISynqNet synqNet,
						   long       port);

MPI_DECL1 long MPI_DECL2
	testPacketNodeClear(MEISynqNet	synqNet,
							long			nodeNumber,
							long			port);

MPI_DECL1 long MPI_DECL2
	testPacketNodeCheck(MEISynqNet	synqNet,
							long			nodeNumber,
							long			port);

#if defined(__cplusplus)
}
#endif

#endif
