/* trace.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_TRACE_H)
#define _TRACE_H

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	The global trace mask is ORed with the object trace mask
 *	to produce a bit mask used to decide on trace output.
 *
 *	The first 16 trace mask bits are identical across all
 *	objects.  The last 16 are object/module-specific.
 *
 *	However, setting any of the (undefined) last 16 mask bits
 *	in the global trace mask will override the object trace
 *	mask.
 */

typedef enum {
	MEITraceNONE = 0,

	MEITraceFIRST = (int)0x0001,

	MEITraceFUNCTION_ENTRY  = (int)(MEITraceFIRST << 0),
	MEITraceFUNCTION_RETURN = (int)(MEITraceFIRST << 1),
	MEITraceMEMORY_ALLOC	= (int)(MEITraceFIRST << 2),
	MEITraceMEMORY_FREE		= (int)(MEITraceFIRST << 3),
	MEITraceMEMORY_GET		= (int)(MEITraceFIRST << 4),
	MEITraceMEMORY_SET		= (int)(MEITraceFIRST << 5),
	MEITraceVALIDATE		= (int)(MEITraceFIRST << 6),
	MEITraceLOCK_GIVE		= (int)(MEITraceFIRST << 7),
	MEITraceLOCK_TAKE		= (int)(MEITraceFIRST << 8),
	MEITraceEVENT			= (int)(MEITraceFIRST << 9),

	MEITraceLAST = (int)(MEITraceFIRST << (((sizeof(int) * 8) / 2) - 1)),

	MEITraceALL = (int)((MEITraceLAST << 1) - 1)
} MEITrace;

typedef	unsigned long MEITraceMask;

#if defined(MEI_TRACE)
#define	meiTrace(mask, format) \
			((mask) && \
			 meiPlatformTrace(format))
#define	meiTrace1(mask, format, arg1) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1)))
#define	meiTrace2(mask, format, arg1, arg2) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2)))
#define	meiTrace3(mask, format, arg1, arg2, arg3) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2), \
								(arg3)))
#define	meiTrace4(mask, format, arg1, arg2, arg3, arg4) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2), \
								(arg3), \
								(arg4)))
#define	meiTrace5(mask, format, arg1, arg2, arg3, arg4, arg5) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2), \
								(arg3), \
								(arg4), \
								(arg5)))
#define	meiTrace6(mask, format, arg1, arg2, arg3, arg4, arg5, arg6) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2), \
								(arg3), \
								(arg4), \
								(arg5), \
								(arg6)))
#define	meiTrace7(mask, format, arg1, arg2, arg3, arg4, arg5, arg6, arg7) \
			((mask) && \
			 meiPlatformTrace((format), \
								(arg1), \
								(arg2), \
								(arg3), \
								(arg4), \
								(arg5), \
								(arg6), \
								(arg7)))
#else
#define meiTrace(mask, format)
#define meiTrace1(mask, format, arg1)
#define meiTrace2(mask, format, arg1, arg2)
#define meiTrace3(mask, format, arg1, arg2, arg3)
#define meiTrace4(mask, format, arg1, arg2, arg3, arg4)
#define meiTrace5(mask, format, arg1, arg2, arg3, arg4, arg5)
#define meiTrace6(mask, format, arg1, arg2, arg3, arg4, arg5, arg6)
#define meiTrace7(mask, format, arg1, arg2, arg3, arg4, arg5, arg6, arg7)
#endif

#include "meidef.h"

typedef long (*MEITraceFunction)(const char *buffer);

/* MPI_DECL1 */ extern MEITraceMask /* MPI_DECL2 */
	MEITraceMaskGLOBAL;

MPI_DECL1 char MPI_DECL2
	meiTraceEol(char eol);

MPI_DECL1 long MPI_DECL2
	meiTraceFile(const char *fileName);

MPI_DECL1 MEITraceFunction MPI_DECL2
	meiTraceFunction(MEITraceFunction traceFunction);

#define	meiTraceGET()	(MEITraceMaskGLOBAL)

MPI_DECL1 MEITraceMask MPI_DECL2
	meiTraceGet(void);

MPI_DECL1 MEITraceMask MPI_DECL2
	meiTraceSet(MEITraceMask mask);

/* 
 * Trace mask text message functions
 */

/* Pointer to function returning module trace messages */
typedef const char *(*MEITraceMsgFunction)(long);

#define	meiTraceMsgID(module,number)	\
			((long)((((module) & MPIModuleIdMAX) << 8) | \
				    ((number) & 0xFF)))

#define	meiTraceMsgMODULE(messageId)	\
			(((messageId) & (MPIModuleIdMAX << 8)) >> 8)

#define	meiTraceMsgNUMBER(messageId)	((messageId) & 0xFF)

#define	meiTraceMsgOBJ_NUM(number) (number - 16)

#define	mpiTraceMsgMASK(messageId)	(MEITraceFIRST << \
			(meiTraceMsgNUMBER(messageId)))

#define	meiTraceMsgMASK(messageId)	((MEITraceLAST << 1) << \
			(meiTraceMsgNUMBER(messageId)))

MPI_DECL1 long MPI_DECL2
	meiTraceMaskBits(long	mask,
					 long	*bitCount,
					 long	*bit);

MPI_DECL1 const char * MPI_DECL2
	meiTraceMsg(long	messageId,
				char	*messageText);

MPI_DECL1 long MPI_DECL2
	meiTraceMsgFunction(MPIModuleId			moduleId,
			 		   MEITraceMsgFunction	function);

#if defined(__cplusplus)
}
#endif

#endif
