/* xmpcan.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_XMPCAN_H)
#define _XMPCAN_H

/* == Macros =============================================================== */
/* Version number for the CAN DPR interface. i.e. The version of this file. */
#define	MEIXmpCanVERSION	(0x0003)

/* MEIXmpCan memory 64K 16-bit words. */
#define	MEIXmpCanMemBASE	    (0x560000)
#define	MEIXmpCanSemaphoreBASE	(0x570000)
#define MEIXmpCanPageSIZE	    (0x400)	
#define MEIXmpCanPages		    (4)
#define MEIXmpCanMemSIZE	    (MEIXmpCanPageSIZE * MEIXmpCanPages)

/* The two codes that are written into the signature field depending if the */
/* firmware or the bootloader are running. */
#define	MEIXmpCanSIGNATURE       (0xC0DE)
#define	MEIXmpCanSTART_SIGNATURE (0xBEAD)

/* Address of the firmware revision. */
#define MEIXmpCanFIRMWARE_REV_ADDR (0x1041)

/* The maximum number of nodes that can exist on a CANopen network. */
/* this is defined in the CANOpen specification DS301. */
#define MEIXmpCanNodeCountMAX   (127)

/* The maxumum number of PDOs that our implementation can use. */
#define MEIXmpCanPDOTransmitMAX (256)
#define MEIXmpCanPDOReceiveMAX  (256)

/* The number of elements in the event buffer. */
#define MEIXmpCanEventBufferSIZE (20)

/* The maximum number of messages that can be logged. */
#define MEIXmpCanMessageLogSIZE	(60)

/* Error codes used internally by the CANApp functions. */
/* These could be returned as the error codes to CanCommands. */
#define MEIXmpCanErrorNONE            0
#define MEIXmpCanErrorSDO_TIMEOUT     1
#define MEIXmpCanErrorSDO_ABORT       2
#define MEIXmpCanErrorSDO_PROTOCOL    3
#define MEIXmpCanErrorTX_OVERFLOW     4
#define MEIXmpCanErrorRTR_TX_OVERFLOW 5
#define MEIXmpCanErrorRX_BUFFER_EMPTY 6
#define MEIXmpCanErrorBUS_OFF         7

/* == Enumerations ========================================================= */

typedef enum {
    MEIXmpCanBitRate1000K = 0,
    MEIXmpCanBitRate800K,
    MEIXmpCanBitRate500K,
    MEIXmpCanBitRate250K,
    MEIXmpCanBitRate125K,
    MEIXmpCanBitRate50K,
    MEIXmpCanBitRate20K,
    MEIXmpCanBitRate10K
} MEIXmpCanBitRate;

typedef enum {
    MEIXmpCanBusStateOFF         = 0,
    MEIXmpCanBusStatePASSIVE     = 1,
    MEIXmpCanBusStateOPERATIONAL = 2
} MEIXmpCanBusState;

typedef enum {
    MEIXmpCanCommandTypeNONE     = 0,
    MEIXmpCanCommandTypeBUS_START,
    MEIXmpCanCommandTypeBUS_STOP,
    MEIXmpCanCommandTypeNMT_ENTER_PRE_OPERATIONAL,
    MEIXmpCanCommandTypeNMT_START_REMOTE_NODE,
    MEIXmpCanCommandTypeSDO_READ,
    MEIXmpCanCommandTypeSDO_WRITE,
    MEIXmpCanCommandTypePDO_SEND,
    MEIXmpCanCommandTypeCLEAR_STATUS_BITS,
    MEIXmpCanCommandTypeSDO_READ_POLL,
    MEIXmpCanCommandTypeSDO_WRITE_POLL,
    MEIXmpCanCommandTypeNMT_STOP_REMOTE_NODE,
    MEIXmpCanCommandTypeNMT_RESET_NODE,
    MEIXmpCanCommandTypeNMT_RESET_COMMUNICATION
} MEIXmpCanCommandType;

typedef enum {
    MEIXmpCanEventMaskBUS_STATE       = (int)0x0001,
    MEIXmpCanEventMaskRECEIVE_OVERRUN = (int)0x0002,
    MEIXmpCanEventMaskEMERGENGY       = (int)0x0004,
    MEIXmpCanEventMaskNODE_BOOT       = (int)0x0008,
    MEIXmpCanEventMaskHEALTH          = (int)0x0010,
    MEIXmpCanEventMaskDIGITAL_INPUT   = (int)0x0020,
    MEIXmpCanEventMaskANALOG_INPUT    = (int)0x0040
} MEIXmpCanEventMask;

typedef enum {
    MEIXmpCanEventTypeNONE = 0,
    MEIXmpCanEventTypeBUS_STATE,
    MEIXmpCanEventTypeRECEIVE_OVERRUN,
    MEIXmpCanEventTypeEMERGENGY,
    MEIXmpCanEventTypeNODE_BOOT,
    MEIXmpCanEventTypeHEALTH,
    MEIXmpCanEventTypeDIGITAL_INPUT,
    MEIXmpCanEventTypeANALOG_INPUT
} MEIXmpCanEventType;

typedef enum {
    MEIXmpCanInterruptNONE      = 0,
    MEIXmpCanInterruptNEW_EVENT = 1
} MEIXmpCanInterrupt;

typedef enum {
    MEIXmpCanHostInterruptNONE     = 0,
    MEIXmpCanHostInterruptRESET    = 1,
    MEIXmpCanHostInterruptBOOTLOAD = 2
} MEIXmpCanHostInterrupt;

typedef enum {
	/* Bits 0 to 6  Reported state of the node */
    MEIXmpCanNodeHealthBOOT_UP         = (int)0x0000,
    MEIXmpCanNodeHealthSTOPPED         = (int)0x0004,
    MEIXmpCanNodeHealthOPERATIONAL     = (int)0x0005, 
    MEIXmpCanNodeHealthPRE_OPERATIONAL = (int)0x007f,
    MEIXmpCanNodeHealthUNKNOWN         = (int)0x0050,  /* This is hopefully unused by any node */
    MEIXmpCanNodeHealthNMT_STATE       = (int)0x007f,
    /* Bit 7        Toggle bit when node guarding. */
    MEIXmpCanNodeHealthTOGGLE_BIT      = (int)0x0080,
    MEIXmpCanNodeHealthLIVE            = (int)0x0100,
    MEIXmpCanNodeHealthGUARDING        = (int)0x0200,
    MEIXmpCanNodeHealthTOGGLE_FAULT    = (int)0x0400,
    MEIXmpCanNodeHealthFLAG            = (int)0x0800,
    MEIXmpCanNodeHealthFIRST_TOGGLE    = (int)0x1000,
	MEIXmpCanNodeHealthRESPONSE_FAULT  = (int)0x2000
} MEIXmpCanNodeHealth;

typedef enum {
    MEIXmpCanNodeTypeNONE = 0,
    MEIXmpCanNodeTypeIO   = 401
} MEIXmpCanNodeType;

typedef enum {
    MEIXmpCanTransmissionTypeCYCLIC = 0,
    MEIXmpCanTransmissionTypeEVENT  = 1
} MEIXmpCanTransmissionType;

typedef enum {
	/* Bits  0 to 10 = Identifier */
    MEIXmpCanPDODataIDENTIFIER_MASK = (int)0x07ff,
    /* Bits 11 to 12 = Type       */
    MEIXmpCanPDODataTYPE_MASK       = (int)0x1800,
    MEIXmpCanPDODataNOT_USED        = (int)0x0000,
    MEIXmpCanPDODataEVENT           = (int)0x0800,
    MEIXmpCanPDODataCYCLIC          = (int)0x1000
    /* Bit 13 to 15 = Length less 1    */
    /* i.e. 0=>1 byte, 7=>8 bytes etc. */
} MEIXmpCanPDOData;

typedef enum {
    MEIXmpCanStatusBitsMaskSW_RX_OVERFLOW = 1,
    MEIXmpCanStatusBitsMaskHW_RX_OVERFLOW = 2,
    MEIXmpCanStatusBitsMaskCONFIG_TEST_FAILED = 4
} MEIXmpCanStatusBitsMask;

/* == Structures =========================================================== */

typedef struct MEIXmpCanNodeConfig {

    /* Bits 0  to 1  RxPDO1 transmission type. */
    /* Bits 2  to 3  RxPDO2 transmission type. */
    /* Bits 4  to 7  Reserved.                 */
    /* Bits 8  to 9  TxPDO1 transmission type. */
    /* Bits 10 to 11 TxPDO2 transmission type. */
    /* Bits 12 to 15 Reserved.                 */
    unsigned int volatile PDOTransmissionTypes;

} MEIXmpCanNodeConfig;

typedef struct MEIXmpCanConfig {

    /* Valid config structure signature.                                 */
    /* The host or Sharc will write the valid signature here once it has */
    /* updated the contents of the config structure.                     */
    unsigned int volatile Valid;

    /* The bit rate that the CAN netwok is using. */
    MEIXmpCanBitRate volatile BitRate;

    /* The period between sending SYNC messages in milli Seconds. */
    unsigned int volatile CyclicPeriod;

    /* The period used for checking the health of nodes.                 */
    /* Specified in ms, a value of zero will disable the health checking */
    /* protocol. */
    unsigned int volatile HealthPeriod;

    /* The controllers CAN node number.*/
    unsigned int volatile NodeNumber;

    /* Selects if the auto bus on mode of the CAN controller is ebabled. */
    /* A 1 indicates that auto bus on mode is selected.                  */
    unsigned int volatile AutoBusOn;

    /* Inhibit time for all PDOs. */
    unsigned int volatile InhibitTime;

    /* The configuration for each of the nodes. */
    /* Note index zero is not used.             */
    MEIXmpCanNodeConfig Node[MEIXmpCanNodeCountMAX+1];

} MEIXmpCanConfig;

typedef struct MEIXmpCanStatus {

    /* The current bus state. */
    MEIXmpCanBusState volatile BusState;

    /* The number of messages received or transmidded per second. */
    unsigned int volatile MessageRate;

    /* Bits 0 to 7 are the receive error counter.                            */
    /* Bits 8 to 15 are the transmit error counter.                          */
    unsigned int volatile ErrorCounters;

    /* Error bits register.                                                  */
    MEIXmpCanStatusBitsMask volatile Bits;

    /* Internal timer on the CAN processor counting in milliseconds. */
    unsigned int volatile Tick;

} MEIXmpCanStatus;

/* The "status" and "info" and other runtime data for each of the nodes. */
/* This is all squeezed into one structure to conserve DPR memory.       */
typedef struct MEIXmpCanNodeData {

    /* The type of this node or MEICanNodeTypeNONE if no node detected. */
    MEIXmpCanNodeType volatile Type; /* = Info */

    MEIXmpCanNodeHealth volatile Health;

    /* Bits 0 to 7  = number of inputs.  */
    /* Bits 8 to 15 = number of outputs. */
    unsigned int volatile DigitalCount;  /* = Info */
    unsigned int volatile AnalogueCount; /* = Info */

    /* Bits 0 to 7  = remote nodes PDO1 (digital) linked to this local PDO.  */
    /* Bits 8 to 15 = remote nodes PDO2 (analogue) linked to this local PDO. */
    unsigned int volatile ReceivePDO;    /* = Runtime data */
    unsigned int volatile TransmitPDO;   /* = Runtime data */

} MEIXmpCanNodeData;

/**
 * MEIXmpCanCommandTypeNONE
 *   There is no active command / the command is acknoledged
 *
 * MEIXmpCanCommandTypeBUS_START
 *   This puts the CAN bus into operational state if it is Bus off.
 *   Transmitted data:
 *     None
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXMpCanCommandTypeBUS_STOP
 *   This puts the CAN bus into bus off state.
 *   Transmitted data:
 *     None
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXMpCanCommandTypeNMT_START
 *   This puts the CAN bus into bus off state.
 *   Transmitted data:
 *     Data[0] = Node, 0 is all nodes
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXMpCanCommandTypeNMT_STOP
 *   This puts the CAN bus into bus off state.
 *   Transmitted data:
 *     Data[0] = Node, 0 is all nodes
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXmpCanCommandTypeSDO_READ
 *   This reads the a remote nodes object dictionary using the SDO protocol.
 *   Transmitted data:
 *     Data[0] = Node
 *     Data[1] = Index
 *     Data[2] = SubIndex
 *     Data[3] = Length
 *   Received data:
 *     Data[0] = Error code.
 *     Data[4] = Low Data word
 *     Data[5] = High Data word
 *
 * MEIXmpCanCommandTypeSDO_READ_POLL,
 *   Checks to see if the SDO response has been received.
 *   Transmitted data:
 *     None
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXmpCanCommandTypeSDO_WRITE
 *   This writes to a remote nodes object dictionary using the SDO protocol.
 *   Transmitted data:
 *     Data[0] = Node
 *     Data[1] = Index
 *     Data[2] = SubIndex
 *     Data[3] = Length
 *     Data[4] = Low Data word
 *     Data[5] = High Data word
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXmpCanCommandTypeSDO_WRITE_POLL
 *   Checks to see if the SDO response has been received.
 *   Transmitted data:
 *     None
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXmpCanCommandTypePDO_SEND
 *   This transmits the supplied PDO
 *   Transmitted data:
 *     Data[0], bits 0 to 7 are the node number.
 *     Data[0], bits 8 to 15 are the PDO number.
 *     Data[1 to 4] = The PDO data to send.
 *     Data[5] = 0 => will always wait, !0 => Disable the wait if the PDO is cyclic.
 *   Received data:
 *     Data[0] = Error code.
 *
 * MEIXmpCanCommandTypeCLEAR_STATUS_BITS,
 *   Clears the selected status bits.
 *   Transmitted data:
 *     Data[0], mask of statust bit to clear.
 *   Received data:
 *     Data[0] = Error code.
 *
 */
typedef struct MEIXmpCanCommand {

	MEIXmpCanCommandType Type;

    unsigned int volatile Data[6];

} MEIXmpCanCommand;

/*
 * MEIXmpCanEventTypeBUS_STATE
 *   The BusState has changed.
 *   Data[0] contains the new bus state.
 *
 * MEIXmpCanEventTypeRECEIVE_OVERRUN
 *   The CAN hardware detected a receive overrun.
 *
 * MEIXmpCanEventTypeEMERGENGY
 *   An emergency message was received from a node.
 *   Data[0] contains the node number.
 *   Data[1 to 4] contains the contents of the emergency message.
 *
 * MEIXmpCanEventTypeNODE_BOOT
 *   A node boot message was received from a node.
 *   Data[0] contains the node number.
 *
 * MEIXmpCanEventTypeHEALTH
 *   The health of a node has changed.
 *   Data[0] contains the node number.
 *   Data[1] contains the new node health.
 *
 * MEIXmpCanEventTypeDIGITAL_INPUT
 *   A digital input event was received from a node.
 *   Data[0] contains the node number.
 *   Data[1 to 4] contains the new input state.
 *
 * MEIXmpCanEventTypeANALOG_INPUT
 *   A analogue input event was received from a node.
 *   Data[0] contains the node number.
 *   Data[1 to 4] contains the new input state.
 */
typedef struct MEIXmpCanEventDescription {

    /* The type of CAN event. */
    MEIXmpCanEventType volatile Type;

    /* Data associated with the event. */
    unsigned int volatile Data[5];

} MEIXmpCanEventDescription;

typedef struct MEIXmpCanEvent {

    /* A mask which will enable or disable the generation of different CAN */
    /* events.                                                             */
    MEIXmpCanEventMask volatile Mask;

    /* The buffer of CAN events. */
    MEIXmpCanEventDescription volatile Buffer[ MEIXmpCanEventBufferSIZE ];

    /* The location within the buffer where the CAN firmware will place the */
    /* next event.  The host routines modify their own tail pointers        */
    /* allowing multiple tasks on the host to all receive these events.     */
    volatile unsigned int Head;

    /* The hosts CAN handle to the object associated with the CAN controller. */
    /* This is done to fit in with the existing event scheme.  It does not    */
    /* allow multiple host can objects to receive events from the 2407.       */
    unsigned int HostCanHandleLo;
    unsigned int HostCanHandleHi;

} MEIXmpCanEvent;

typedef struct MEIXmpCanPDO {

    /* The contents of the PDO. */
    unsigned int volatile Data[4];

} MEIXmpCanPDO;

typedef struct MEIXmpCANMessage {
    unsigned int Identifier;
    unsigned int Length;
    unsigned int Data[4];
} MEIXmpCANMessage;

typedef struct MEIXmpCanMessageLogEntry {
	unsigned int Tick;
	unsigned int Counter;
	MEIXmpCANMessage Message;
} MEIXmpCanMessageLogEntry;

typedef struct MEIXmpCanMessageLog {
	/* 0 will enable the message logger, !0 will stop the message logging. */
	unsigned int Stop;
	unsigned int Head;
	MEIXmpCanMessageLogEntry Data[MEIXmpCanMessageLogSIZE];
} MEIXmpCanMessageLog;

typedef struct MEIXmpCanTimings {
	unsigned int MailboxISR;
	unsigned int TimerISR;
	unsigned int MainLoopStartCommand;
	unsigned int MainLoopEnd;
} MEIXmpCanTimings;

/* MEICanDPR defines the complete memory map of the DPR interface between */
/* the Host and the 2407 CAN processor.                                   */
typedef struct MEIXmpCan {

    /* A location that is repeatedly updated with "MEIXmpCanSIGNATURE".    */
    /* If the host clears this location and it is refereshed with          */
    /* "MEIXmpCanSIGNATURE" then the host can assume that the CAN firmware */
    /* is executing.                                                       */
    unsigned int volatile Signature;

    /* A copy of the CAN bootloaders version. */
    unsigned int volatile BootVersion;

    /* A copy of the CAN firmware interface version "MEIXmpCanVERSION". */
    unsigned int volatile FirmwareVersion;

    /* A copy of the CAN firmware revision.                      */
    /* Bits 0 to 2 = Sub-Revision (number between 1 and 7)       */
    /* Bits 3 to 9 = Revision (character, 0=>A, 1=>B, 2=>C etc.) */
    unsigned int volatile FirmwareRevision;

    MEIXmpCanConfig     Config;
    MEIXmpCanStatus     Status;

    /* Note - index zero is not used. */
    MEIXmpCanNodeData   NodeData[MEIXmpCanNodeCountMAX+1];

    MEIXmpCanCommand    Command;
    MEIXmpCanEvent      Event;
    MEIXmpCanPDO        ReceivePDO[MEIXmpCanPDOReceiveMAX];
    
    MEIXmpCanPDO              TransmitPDO[MEIXmpCanPDOTransmitMAX];
    unsigned int volatile     InhibitTimers[MEIXmpCanPDOTransmitMAX];
    MEIXmpCanPDOData volatile TransmitPDOData[MEIXmpCanPDOTransmitMAX];

	MEIXmpCanMessageLog MessageLog;
    
	MEIXmpCanTimings Timings;

	/* Unused space in the DPR.
	   The size of this must always be adjjsted so that this structure always 
	   matches to size of the DPR.  This makes sure the last two location align with the 
	  interrupt generating locations of the DPR. */
    unsigned int Unused[ 0x04 ];

    /* Force a bootload flag. */
    unsigned int volatile ForceBoot;

    /* The CAN controller writting to this addres will generate an interrupt */
    /* on the host.  The host clears the interrupt by reading or writing     */
    /* this address.                                                         */
    MEIXmpCanInterrupt volatile CAN2HostInterrupt;
    
    /* The host writting to this address will generate an interrupt on the */
    /* CAN controller.  The CAN controller clears the interrupt by reading */
    /* or writing this address.                                            */
    MEIXmpCanHostInterrupt volatile Host2CANInterrupt;

} MEIXmpCan;

/* This is a map of the DPR semaphores in the DPR connected to the 2407. */
typedef struct MEIXmpCanSemaphore {

    unsigned int volatile Flag[8];

} MEIXmpCanSemaphore;

#endif  /* _XMPCAN_H */
